# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: keyspace/Keyspace.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='keyspace/Keyspace.proto',
  package='keyspace',
  syntax='proto3',
  serialized_options=_b('\n\027grakn.protocol.keyspaceB\rKeyspaceProto'),
  serialized_pb=_b('\n\x17keyspace/Keyspace.proto\x12\x08keyspace\"\xd6\x01\n\x08Keyspace\x1a\x66\n\x08Retrieve\x1a=\n\x03Req\x12\x1a\n\x08username\x18\x01 \x01(\tR\x08username\x12\x1a\n\x08password\x18\x02 \x01(\tR\x08password\x1a\x1b\n\x03Res\x12\x14\n\x05names\x18\x01 \x03(\tR\x05names\x1a\x62\n\x06\x44\x65lete\x1aQ\n\x03Req\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x1a\n\x08username\x18\x02 \x01(\tR\x08username\x12\x1a\n\x08password\x18\x03 \x01(\tR\x08password\x1a\x05\n\x03Res2\xa7\x01\n\x0fKeyspaceService\x12L\n\x08retrieve\x12\x1f.keyspace.Keyspace.Retrieve.Req\x1a\x1f.keyspace.Keyspace.Retrieve.Res\x12\x46\n\x06\x64\x65lete\x12\x1d.keyspace.Keyspace.Delete.Req\x1a\x1d.keyspace.Keyspace.Delete.ResB(\n\x17grakn.protocol.keyspaceB\rKeyspaceProtob\x06proto3')
)




_KEYSPACE_RETRIEVE_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='keyspace.Keyspace.Retrieve.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='username', full_name='keyspace.Keyspace.Retrieve.Req.username', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='username', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='password', full_name='keyspace.Keyspace.Retrieve.Req.password', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='password', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=62,
  serialized_end=123,
)

_KEYSPACE_RETRIEVE_RES = _descriptor.Descriptor(
  name='Res',
  full_name='keyspace.Keyspace.Retrieve.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='names', full_name='keyspace.Keyspace.Retrieve.Res.names', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='names', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=125,
  serialized_end=152,
)

_KEYSPACE_RETRIEVE = _descriptor.Descriptor(
  name='Retrieve',
  full_name='keyspace.Keyspace.Retrieve',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_KEYSPACE_RETRIEVE_REQ, _KEYSPACE_RETRIEVE_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=50,
  serialized_end=152,
)

_KEYSPACE_DELETE_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='keyspace.Keyspace.Delete.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='keyspace.Keyspace.Delete.Req.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='name', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='username', full_name='keyspace.Keyspace.Delete.Req.username', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='username', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='password', full_name='keyspace.Keyspace.Delete.Req.password', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='password', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=164,
  serialized_end=245,
)

_KEYSPACE_DELETE_RES = _descriptor.Descriptor(
  name='Res',
  full_name='keyspace.Keyspace.Delete.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=125,
  serialized_end=130,
)

_KEYSPACE_DELETE = _descriptor.Descriptor(
  name='Delete',
  full_name='keyspace.Keyspace.Delete',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_KEYSPACE_DELETE_REQ, _KEYSPACE_DELETE_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=154,
  serialized_end=252,
)

_KEYSPACE = _descriptor.Descriptor(
  name='Keyspace',
  full_name='keyspace.Keyspace',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_KEYSPACE_RETRIEVE, _KEYSPACE_DELETE, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=38,
  serialized_end=252,
)

_KEYSPACE_RETRIEVE_REQ.containing_type = _KEYSPACE_RETRIEVE
_KEYSPACE_RETRIEVE_RES.containing_type = _KEYSPACE_RETRIEVE
_KEYSPACE_RETRIEVE.containing_type = _KEYSPACE
_KEYSPACE_DELETE_REQ.containing_type = _KEYSPACE_DELETE
_KEYSPACE_DELETE_RES.containing_type = _KEYSPACE_DELETE
_KEYSPACE_DELETE.containing_type = _KEYSPACE
DESCRIPTOR.message_types_by_name['Keyspace'] = _KEYSPACE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Keyspace = _reflection.GeneratedProtocolMessageType('Keyspace', (_message.Message,), dict(

  Retrieve = _reflection.GeneratedProtocolMessageType('Retrieve', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _KEYSPACE_RETRIEVE_REQ,
      __module__ = 'keyspace.Keyspace_pb2'
      # @@protoc_insertion_point(class_scope:keyspace.Keyspace.Retrieve.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _KEYSPACE_RETRIEVE_RES,
      __module__ = 'keyspace.Keyspace_pb2'
      # @@protoc_insertion_point(class_scope:keyspace.Keyspace.Retrieve.Res)
      ))
    ,
    DESCRIPTOR = _KEYSPACE_RETRIEVE,
    __module__ = 'keyspace.Keyspace_pb2'
    # @@protoc_insertion_point(class_scope:keyspace.Keyspace.Retrieve)
    ))
  ,

  Delete = _reflection.GeneratedProtocolMessageType('Delete', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _KEYSPACE_DELETE_REQ,
      __module__ = 'keyspace.Keyspace_pb2'
      # @@protoc_insertion_point(class_scope:keyspace.Keyspace.Delete.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _KEYSPACE_DELETE_RES,
      __module__ = 'keyspace.Keyspace_pb2'
      # @@protoc_insertion_point(class_scope:keyspace.Keyspace.Delete.Res)
      ))
    ,
    DESCRIPTOR = _KEYSPACE_DELETE,
    __module__ = 'keyspace.Keyspace_pb2'
    # @@protoc_insertion_point(class_scope:keyspace.Keyspace.Delete)
    ))
  ,
  DESCRIPTOR = _KEYSPACE,
  __module__ = 'keyspace.Keyspace_pb2'
  # @@protoc_insertion_point(class_scope:keyspace.Keyspace)
  ))
_sym_db.RegisterMessage(Keyspace)
_sym_db.RegisterMessage(Keyspace.Retrieve)
_sym_db.RegisterMessage(Keyspace.Retrieve.Req)
_sym_db.RegisterMessage(Keyspace.Retrieve.Res)
_sym_db.RegisterMessage(Keyspace.Delete)
_sym_db.RegisterMessage(Keyspace.Delete.Req)
_sym_db.RegisterMessage(Keyspace.Delete.Res)


DESCRIPTOR._options = None

_KEYSPACESERVICE = _descriptor.ServiceDescriptor(
  name='KeyspaceService',
  full_name='keyspace.KeyspaceService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=255,
  serialized_end=422,
  methods=[
  _descriptor.MethodDescriptor(
    name='retrieve',
    full_name='keyspace.KeyspaceService.retrieve',
    index=0,
    containing_service=None,
    input_type=_KEYSPACE_RETRIEVE_REQ,
    output_type=_KEYSPACE_RETRIEVE_RES,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='delete',
    full_name='keyspace.KeyspaceService.delete',
    index=1,
    containing_service=None,
    input_type=_KEYSPACE_DELETE_REQ,
    output_type=_KEYSPACE_DELETE_RES,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_KEYSPACESERVICE)

DESCRIPTOR.services_by_name['KeyspaceService'] = _KEYSPACESERVICE

# @@protoc_insertion_point(module_scope)
