# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: session/Concept.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='session/Concept.proto',
  package='session',
  syntax='proto3',
  serialized_options=_b('\n\026grakn.protocol.sessionB\014ConceptProto'),
  serialized_pb=_b('\n\x15session/Concept.proto\x12\x07session\"\xd2J\n\x06Method\x1a\x98\x1f\n\x03Req\x12K\n\x12\x63oncept_delete_req\x18\x64 \x01(\x0b\x32\x1b.session.Concept.Delete.ReqH\x00R\x10\x63onceptDeleteReq\x12j\n\x1cschemaConcept_isImplicit_req\x18\xc8\x01 \x01(\x0b\x32%.session.SchemaConcept.IsImplicit.ReqH\x00R\x1aschemaConceptIsImplicitReq\x12\x64\n\x1aschemaConcept_getLabel_req\x18\xc9\x01 \x01(\x0b\x32#.session.SchemaConcept.GetLabel.ReqH\x00R\x18schemaConceptGetLabelReq\x12\x64\n\x1aschemaConcept_setLabel_req\x18\xca\x01 \x01(\x0b\x32#.session.SchemaConcept.SetLabel.ReqH\x00R\x18schemaConceptSetLabelReq\x12^\n\x18schemaConcept_getSup_req\x18\xcb\x01 \x01(\x0b\x32!.session.SchemaConcept.GetSup.ReqH\x00R\x16schemaConceptGetSupReq\x12^\n\x18schemaConcept_setSup_req\x18\xcc\x01 \x01(\x0b\x32!.session.SchemaConcept.SetSup.ReqH\x00R\x16schemaConceptSetSupReq\x12X\n\x16schemaConcept_sups_req\x18\xcd\x01 \x01(\x0b\x32\x1f.session.SchemaConcept.Sups.ReqH\x00R\x14schemaConceptSupsReq\x12X\n\x16schemaConcept_subs_req\x18\xce\x01 \x01(\x0b\x32\x1f.session.SchemaConcept.Subs.ReqH\x00R\x14schemaConceptSubsReq\x12=\n\rrule_when_req\x18\xac\x02 \x01(\x0b\x32\x16.session.Rule.When.ReqH\x00R\x0bruleWhenReq\x12=\n\rrule_then_req\x18\xad\x02 \x01(\x0b\x32\x16.session.Rule.Then.ReqH\x00R\x0bruleThenReq\x12L\n\x12role_relations_req\x18\x91\x03 \x01(\x0b\x32\x1b.session.Role.Relations.ReqH\x00R\x10roleRelationsReq\x12\x46\n\x10role_players_req\x18\x92\x03 \x01(\x0b\x32\x19.session.Role.Players.ReqH\x00R\x0erolePlayersReq\x12O\n\x13type_isAbstract_req\x18\xf4\x03 \x01(\x0b\x32\x1c.session.Type.IsAbstract.ReqH\x00R\x11typeIsAbstractReq\x12R\n\x14type_setAbstract_req\x18\xf5\x03 \x01(\x0b\x32\x1d.session.Type.SetAbstract.ReqH\x00R\x12typeSetAbstractReq\x12L\n\x12type_instances_req\x18\xf6\x03 \x01(\x0b\x32\x1b.session.Type.Instances.ReqH\x00R\x10typeInstancesReq\x12=\n\rtype_keys_req\x18\xf7\x03 \x01(\x0b\x32\x16.session.Type.Keys.ReqH\x00R\x0btypeKeysReq\x12O\n\x13type_attributes_req\x18\xf8\x03 \x01(\x0b\x32\x1c.session.Type.Attributes.ReqH\x00R\x11typeAttributesReq\x12\x46\n\x10type_playing_req\x18\xf9\x03 \x01(\x0b\x32\x19.session.Type.Playing.ReqH\x00R\x0etypePlayingReq\x12:\n\x0ctype_has_req\x18\xfa\x03 \x01(\x0b\x32\x15.session.Type.Has.ReqH\x00R\ntypeHasReq\x12:\n\x0ctype_key_req\x18\xfb\x03 \x01(\x0b\x32\x15.session.Type.Key.ReqH\x00R\ntypeKeyReq\x12@\n\x0etype_plays_req\x18\xfc\x03 \x01(\x0b\x32\x17.session.Type.Plays.ReqH\x00R\x0ctypePlaysReq\x12@\n\x0etype_unhas_req\x18\xfd\x03 \x01(\x0b\x32\x17.session.Type.Unhas.ReqH\x00R\x0ctypeUnhasReq\x12@\n\x0etype_unkey_req\x18\xfe\x03 \x01(\x0b\x32\x17.session.Type.Unkey.ReqH\x00R\x0ctypeUnkeyReq\x12\x43\n\x0ftype_unplay_req\x18\xff\x03 \x01(\x0b\x32\x18.session.Type.Unplay.ReqH\x00R\rtypeUnplayReq\x12U\n\x15\x65ntityType_create_req\x18\xd8\x04 \x01(\x0b\x32\x1e.session.EntityType.Create.ReqH\x00R\x13\x65ntityTypeCreateReq\x12[\n\x17relationType_create_req\x18\xbc\x05 \x01(\x0b\x32 .session.RelationType.Create.ReqH\x00R\x15relationTypeCreateReq\x12X\n\x16relationType_roles_req\x18\xbd\x05 \x01(\x0b\x32\x1f.session.RelationType.Roles.ReqH\x00R\x14relationTypeRolesReq\x12^\n\x18relationType_relates_req\x18\xbe\x05 \x01(\x0b\x32!.session.RelationType.Relates.ReqH\x00R\x16relationTypeRelatesReq\x12\x61\n\x19relationType_unrelate_req\x18\xbf\x05 \x01(\x0b\x32\".session.RelationType.Unrelate.ReqH\x00R\x17relationTypeUnrelateReq\x12^\n\x18\x61ttributeType_create_req\x18\xa0\x06 \x01(\x0b\x32!.session.AttributeType.Create.ReqH\x00R\x16\x61ttributeTypeCreateReq\x12g\n\x1b\x61ttributeType_attribute_req\x18\xa1\x06 \x01(\x0b\x32$.session.AttributeType.Attribute.ReqH\x00R\x19\x61ttributeTypeAttributeReq\x12\x64\n\x1a\x61ttributeType_dataType_req\x18\xa2\x06 \x01(\x0b\x32#.session.AttributeType.DataType.ReqH\x00R\x18\x61ttributeTypeDataTypeReq\x12\x64\n\x1a\x61ttributeType_getRegex_req\x18\xa3\x06 \x01(\x0b\x32#.session.AttributeType.GetRegex.ReqH\x00R\x18\x61ttributeTypeGetRegexReq\x12\x64\n\x1a\x61ttributeType_setRegex_req\x18\xa4\x06 \x01(\x0b\x32#.session.AttributeType.SetRegex.ReqH\x00R\x18\x61ttributeTypeSetRegexReq\x12@\n\x0ething_type_req\x18\x84\x07 \x01(\x0b\x32\x17.session.Thing.Type.ReqH\x00R\x0cthingTypeReq\x12R\n\x14thing_isInferred_req\x18\x85\x07 \x01(\x0b\x32\x1d.session.Thing.IsInferred.ReqH\x00R\x12thingIsInferredReq\x12@\n\x0ething_keys_req\x18\x86\x07 \x01(\x0b\x32\x17.session.Thing.Keys.ReqH\x00R\x0cthingKeysReq\x12R\n\x14thing_attributes_req\x18\x87\x07 \x01(\x0b\x32\x1d.session.Thing.Attributes.ReqH\x00R\x12thingAttributesReq\x12O\n\x13thing_relations_req\x18\x88\x07 \x01(\x0b\x32\x1c.session.Thing.Relations.ReqH\x00R\x11thingRelationsReq\x12\x43\n\x0fthing_roles_req\x18\x89\x07 \x01(\x0b\x32\x18.session.Thing.Roles.ReqH\x00R\rthingRolesReq\x12\x46\n\x10thing_relhas_req\x18\x8a\x07 \x01(\x0b\x32\x19.session.Thing.Relhas.ReqH\x00R\x0ethingRelhasReq\x12\x43\n\x0fthing_unhas_req\x18\x8b\x07 \x01(\x0b\x32\x18.session.Thing.Unhas.ReqH\x00R\rthingUnhasReq\x12g\n\x1brelation_rolePlayersMap_req\x18\xe8\x07 \x01(\x0b\x32$.session.Relation.RolePlayersMap.ReqH\x00R\x19relationRolePlayersMapReq\x12^\n\x18relation_rolePlayers_req\x18\xe9\x07 \x01(\x0b\x32!.session.Relation.RolePlayers.ReqH\x00R\x16relationRolePlayersReq\x12O\n\x13relation_assign_req\x18\xea\x07 \x01(\x0b\x32\x1c.session.Relation.Assign.ReqH\x00R\x11relationAssignReq\x12U\n\x15relation_unassign_req\x18\xeb\x07 \x01(\x0b\x32\x1e.session.Relation.Unassign.ReqH\x00R\x13relationUnassignReq\x12O\n\x13\x61ttribute_value_req\x18\xcc\x08 \x01(\x0b\x32\x1c.session.Attribute.Value.ReqH\x00R\x11\x61ttributeValueReq\x12R\n\x14\x61ttribute_owners_req\x18\xcd\x08 \x01(\x0b\x32\x1d.session.Attribute.Owners.ReqH\x00R\x12\x61ttributeOwnersReqB\x05\n\x03req\x1a\xc8\x1f\n\x03Res\x12K\n\x12\x63oncept_delete_res\x18\x64 \x01(\x0b\x32\x1b.session.Concept.Delete.ResH\x00R\x10\x63onceptDeleteRes\x12j\n\x1cschemaConcept_isImplicit_res\x18\xc8\x01 \x01(\x0b\x32%.session.SchemaConcept.IsImplicit.ResH\x00R\x1aschemaConceptIsImplicitRes\x12\x64\n\x1aschemaConcept_getLabel_res\x18\xc9\x01 \x01(\x0b\x32#.session.SchemaConcept.GetLabel.ResH\x00R\x18schemaConceptGetLabelRes\x12\x64\n\x1aschemaConcept_setLabel_res\x18\xca\x01 \x01(\x0b\x32#.session.SchemaConcept.SetLabel.ResH\x00R\x18schemaConceptSetLabelRes\x12^\n\x18schemaConcept_getSup_res\x18\xcb\x01 \x01(\x0b\x32!.session.SchemaConcept.GetSup.ResH\x00R\x16schemaConceptGetSupRes\x12^\n\x18schemaConcept_setSup_res\x18\xcc\x01 \x01(\x0b\x32!.session.SchemaConcept.SetSup.ResH\x00R\x16schemaConceptSetSupRes\x12[\n\x17schemaConcept_sups_iter\x18\xcd\x01 \x01(\x0b\x32 .session.SchemaConcept.Sups.IterH\x00R\x15schemaConceptSupsIter\x12[\n\x17schemaConcept_subs_iter\x18\xce\x01 \x01(\x0b\x32 .session.SchemaConcept.Subs.IterH\x00R\x15schemaConceptSubsIter\x12=\n\rrule_when_res\x18\xac\x02 \x01(\x0b\x32\x16.session.Rule.When.ResH\x00R\x0bruleWhenRes\x12=\n\rrule_then_res\x18\xad\x02 \x01(\x0b\x32\x16.session.Rule.Then.ResH\x00R\x0bruleThenRes\x12O\n\x13role_relations_iter\x18\x91\x03 \x01(\x0b\x32\x1c.session.Role.Relations.IterH\x00R\x11roleRelationsIter\x12I\n\x11role_players_iter\x18\x92\x03 \x01(\x0b\x32\x1a.session.Role.Players.IterH\x00R\x0frolePlayersIter\x12O\n\x13type_isAbstract_res\x18\xf4\x03 \x01(\x0b\x32\x1c.session.Type.IsAbstract.ResH\x00R\x11typeIsAbstractRes\x12R\n\x14type_setAbstract_res\x18\xf5\x03 \x01(\x0b\x32\x1d.session.Type.SetAbstract.ResH\x00R\x12typeSetAbstractRes\x12O\n\x13type_instances_iter\x18\xf6\x03 \x01(\x0b\x32\x1c.session.Type.Instances.IterH\x00R\x11typeInstancesIter\x12@\n\x0etype_keys_iter\x18\xf7\x03 \x01(\x0b\x32\x17.session.Type.Keys.IterH\x00R\x0ctypeKeysIter\x12R\n\x14type_attributes_iter\x18\xf8\x03 \x01(\x0b\x32\x1d.session.Type.Attributes.IterH\x00R\x12typeAttributesIter\x12I\n\x11type_playing_iter\x18\xf9\x03 \x01(\x0b\x32\x1a.session.Type.Playing.IterH\x00R\x0ftypePlayingIter\x12:\n\x0ctype_has_res\x18\xfa\x03 \x01(\x0b\x32\x15.session.Type.Has.ResH\x00R\ntypeHasRes\x12:\n\x0ctype_key_res\x18\xfb\x03 \x01(\x0b\x32\x15.session.Type.Key.ResH\x00R\ntypeKeyRes\x12@\n\x0etype_plays_res\x18\xfc\x03 \x01(\x0b\x32\x17.session.Type.Plays.ResH\x00R\x0ctypePlaysRes\x12@\n\x0etype_unhas_res\x18\xfd\x03 \x01(\x0b\x32\x17.session.Type.Unhas.ResH\x00R\x0ctypeUnhasRes\x12@\n\x0etype_unkey_res\x18\xfe\x03 \x01(\x0b\x32\x17.session.Type.Unkey.ResH\x00R\x0ctypeUnkeyRes\x12\x43\n\x0ftype_unplay_res\x18\xff\x03 \x01(\x0b\x32\x18.session.Type.Unplay.ResH\x00R\rtypeUnplayRes\x12U\n\x15\x65ntityType_create_res\x18\xd8\x04 \x01(\x0b\x32\x1e.session.EntityType.Create.ResH\x00R\x13\x65ntityTypeCreateRes\x12[\n\x17relationType_create_res\x18\xbc\x05 \x01(\x0b\x32 .session.RelationType.Create.ResH\x00R\x15relationTypeCreateRes\x12[\n\x17relationType_roles_iter\x18\xbd\x05 \x01(\x0b\x32 .session.RelationType.Roles.IterH\x00R\x15relationTypeRolesIter\x12^\n\x18relationType_relates_res\x18\xbe\x05 \x01(\x0b\x32!.session.RelationType.Relates.ResH\x00R\x16relationTypeRelatesRes\x12\x61\n\x19relationType_unrelate_res\x18\xbf\x05 \x01(\x0b\x32\".session.RelationType.Unrelate.ResH\x00R\x17relationTypeUnrelateRes\x12^\n\x18\x61ttributeType_create_res\x18\xa0\x06 \x01(\x0b\x32!.session.AttributeType.Create.ResH\x00R\x16\x61ttributeTypeCreateRes\x12g\n\x1b\x61ttributeType_attribute_res\x18\xa1\x06 \x01(\x0b\x32$.session.AttributeType.Attribute.ResH\x00R\x19\x61ttributeTypeAttributeRes\x12\x64\n\x1a\x61ttributeType_dataType_res\x18\xa2\x06 \x01(\x0b\x32#.session.AttributeType.DataType.ResH\x00R\x18\x61ttributeTypeDataTypeRes\x12\x64\n\x1a\x61ttributeType_getRegex_res\x18\xa3\x06 \x01(\x0b\x32#.session.AttributeType.GetRegex.ResH\x00R\x18\x61ttributeTypeGetRegexRes\x12\x64\n\x1a\x61ttributeType_setRegex_res\x18\xa4\x06 \x01(\x0b\x32#.session.AttributeType.SetRegex.ResH\x00R\x18\x61ttributeTypeSetRegexRes\x12@\n\x0ething_type_res\x18\x84\x07 \x01(\x0b\x32\x17.session.Thing.Type.ResH\x00R\x0cthingTypeRes\x12R\n\x14thing_isInferred_res\x18\x85\x07 \x01(\x0b\x32\x1d.session.Thing.IsInferred.ResH\x00R\x12thingIsInferredRes\x12\x43\n\x0fthing_keys_iter\x18\x86\x07 \x01(\x0b\x32\x18.session.Thing.Keys.IterH\x00R\rthingKeysIter\x12U\n\x15thing_attributes_iter\x18\x87\x07 \x01(\x0b\x32\x1e.session.Thing.Attributes.IterH\x00R\x13thingAttributesIter\x12R\n\x14thing_relations_iter\x18\x88\x07 \x01(\x0b\x32\x1d.session.Thing.Relations.IterH\x00R\x12thingRelationsIter\x12\x46\n\x10thing_roles_iter\x18\x89\x07 \x01(\x0b\x32\x19.session.Thing.Roles.IterH\x00R\x0ethingRolesIter\x12\x46\n\x10thing_relhas_res\x18\x8a\x07 \x01(\x0b\x32\x19.session.Thing.Relhas.ResH\x00R\x0ethingRelhasRes\x12\x43\n\x0fthing_unhas_res\x18\x8b\x07 \x01(\x0b\x32\x18.session.Thing.Unhas.ResH\x00R\rthingUnhasRes\x12j\n\x1crelation_rolePlayersMap_iter\x18\xe8\x07 \x01(\x0b\x32%.session.Relation.RolePlayersMap.IterH\x00R\x1arelationRolePlayersMapIter\x12\x61\n\x19relation_rolePlayers_iter\x18\xe9\x07 \x01(\x0b\x32\".session.Relation.RolePlayers.IterH\x00R\x17relationRolePlayersIter\x12O\n\x13relation_assign_res\x18\xea\x07 \x01(\x0b\x32\x1c.session.Relation.Assign.ResH\x00R\x11relationAssignRes\x12U\n\x15relation_unassign_res\x18\xeb\x07 \x01(\x0b\x32\x1e.session.Relation.Unassign.ResH\x00R\x13relationUnassignRes\x12O\n\x13\x61ttribute_value_res\x18\xcc\x08 \x01(\x0b\x32\x1c.session.Attribute.Value.ResH\x00R\x11\x61ttributeValueRes\x12U\n\x15\x61ttribute_owners_iter\x18\xcd\x08 \x01(\x0b\x32\x1e.session.Attribute.Owners.IterH\x00R\x13\x61ttributeOwnersIterB\x05\n\x03res\x1a\xe1\x0b\n\x04Iter\x1a\xd8\x0b\n\x03Res\x12\x64\n\x1bschemaConcept_sups_iter_res\x18\xcd\x01 \x01(\x0b\x32$.session.SchemaConcept.Sups.Iter.ResR\x18schemaConceptSupsIterRes\x12\x64\n\x1bschemaConcept_subs_iter_res\x18\xce\x01 \x01(\x0b\x32$.session.SchemaConcept.Subs.Iter.ResR\x18schemaConceptSubsIterRes\x12X\n\x17role_relations_iter_res\x18\x91\x03 \x01(\x0b\x32 .session.Role.Relations.Iter.ResR\x14roleRelationsIterRes\x12R\n\x15role_players_iter_res\x18\x92\x03 \x01(\x0b\x32\x1e.session.Role.Players.Iter.ResR\x12rolePlayersIterRes\x12X\n\x17type_instances_iter_res\x18\xf6\x03 \x01(\x0b\x32 .session.Type.Instances.Iter.ResR\x14typeInstancesIterRes\x12I\n\x12type_keys_iter_res\x18\xf7\x03 \x01(\x0b\x32\x1b.session.Type.Keys.Iter.ResR\x0ftypeKeysIterRes\x12[\n\x18type_attributes_iter_res\x18\xf8\x03 \x01(\x0b\x32!.session.Type.Attributes.Iter.ResR\x15typeAttributesIterRes\x12R\n\x15type_playing_iter_res\x18\xf9\x03 \x01(\x0b\x32\x1e.session.Type.Playing.Iter.ResR\x12typePlayingIterRes\x12\x64\n\x1brelationType_roles_iter_res\x18\xbd\x05 \x01(\x0b\x32$.session.RelationType.Roles.Iter.ResR\x18relationTypeRolesIterRes\x12L\n\x13thing_keys_iter_res\x18\x86\x07 \x01(\x0b\x32\x1c.session.Thing.Keys.Iter.ResR\x10thingKeysIterRes\x12^\n\x19thing_attributes_iter_res\x18\x87\x07 \x01(\x0b\x32\".session.Thing.Attributes.Iter.ResR\x16thingAttributesIterRes\x12[\n\x18thing_relations_iter_res\x18\x88\x07 \x01(\x0b\x32!.session.Thing.Relations.Iter.ResR\x15thingRelationsIterRes\x12O\n\x14thing_roles_iter_res\x18\x89\x07 \x01(\x0b\x32\x1d.session.Thing.Roles.Iter.ResR\x11thingRolesIterRes\x12s\n relation_rolePlayersMap_iter_res\x18\xe8\x07 \x01(\x0b\x32).session.Relation.RolePlayersMap.Iter.ResR\x1drelationRolePlayersMapIterRes\x12j\n\x1drelation_rolePlayers_iter_res\x18\xe9\x07 \x01(\x0b\x32&.session.Relation.RolePlayers.Iter.ResR\x1arelationRolePlayersIterRes\x12^\n\x19\x61ttribute_owners_iter_res\x18\xcd\x08 \x01(\x0b\x32\".session.Attribute.Owners.Iter.ResR\x16\x61ttributeOwnersIterRes\"\x06\n\x04Null\"\xfb\x01\n\x07\x43oncept\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x36\n\x08\x62\x61seType\x18\x02 \x01(\x0e\x32\x1a.session.Concept.BASE_TYPER\x08\x62\x61seType\x1a\x16\n\x06\x44\x65lete\x1a\x05\n\x03Req\x1a\x05\n\x03Res\"\x8f\x01\n\tBASE_TYPE\x12\r\n\tMETA_TYPE\x10\x00\x12\x0f\n\x0b\x45NTITY_TYPE\x10\x01\x12\x11\n\rRELATION_TYPE\x10\x02\x12\x12\n\x0e\x41TTRIBUTE_TYPE\x10\x03\x12\x08\n\x04ROLE\x10\x04\x12\x08\n\x04RULE\x10\x05\x12\n\n\x06\x45NTITY\x10\x06\x12\x0c\n\x08RELATION\x10\x07\x12\r\n\tATTRIBUTE\x10\x08\"\xc1\x04\n\rSchemaConcept\x1a.\n\x08GetLabel\x1a\x05\n\x03Req\x1a\x1b\n\x03Res\x12\x14\n\x05label\x18\x01 \x01(\tR\x05label\x1a.\n\x08SetLabel\x1a\x1b\n\x03Req\x12\x14\n\x05label\x18\x01 \x01(\tR\x05label\x1a\x05\n\x03Res\x1a\x36\n\nIsImplicit\x1a\x05\n\x03Req\x1a!\n\x03Res\x12\x1a\n\x08implicit\x18\x01 \x01(\x08R\x08implicit\x1a|\n\x06GetSup\x1a\x05\n\x03Req\x1ak\n\x03Res\x12\x38\n\rschemaConcept\x18\x01 \x01(\x0b\x32\x10.session.ConceptH\x00R\rschemaConcept\x12#\n\x04null\x18\x02 \x01(\x0b\x32\r.session.NullH\x00R\x04nullB\x05\n\x03res\x1aN\n\x06SetSup\x1a=\n\x03Req\x12\x36\n\rschemaConcept\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\rschemaConcept\x1a\x05\n\x03Res\x1a\x64\n\x04Sups\x1a\x05\n\x03Req\x1aU\n\x04Iter\x12\x0e\n\x02id\x18\x01 \x01(\x05R\x02id\x1a=\n\x03Res\x12\x36\n\rschemaConcept\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\rschemaConcept\x1a\x64\n\x04Subs\x1a\x05\n\x03Req\x1aU\n\x04Iter\x12\x0e\n\x02id\x18\x01 \x01(\x05R\x02id\x1a=\n\x03Res\x12\x36\n\rschemaConcept\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\rschemaConcept\"\xc2\x01\n\x04Rule\x1a\\\n\x04When\x1a\x05\n\x03Req\x1aM\n\x03Res\x12\x1a\n\x07pattern\x18\x01 \x01(\tH\x00R\x07pattern\x12#\n\x04null\x18\x02 \x01(\x0b\x32\r.session.NullH\x00R\x04nullB\x05\n\x03res\x1a\\\n\x04Then\x1a\x05\n\x03Req\x1aM\n\x03Res\x12\x1a\n\x07pattern\x18\x01 \x01(\tH\x00R\x07pattern\x12#\n\x04null\x18\x02 \x01(\x0b\x32\r.session.NullH\x00R\x04nullB\x05\n\x03res\"\xc6\x01\n\x04Role\x1ag\n\tRelations\x1a\x05\n\x03Req\x1aS\n\x04Iter\x12\x0e\n\x02id\x18\x01 \x01(\x05R\x02id\x1a;\n\x03Res\x12\x34\n\x0crelationType\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\x0crelationType\x1aU\n\x07Players\x1a\x05\n\x03Req\x1a\x43\n\x04Iter\x12\x0e\n\x02id\x18\x01 \x01(\x05R\x02id\x1a+\n\x03Res\x12$\n\x04type\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\x04type\"\xae\x07\n\x04Type\x1a\x36\n\nIsAbstract\x1a\x05\n\x03Req\x1a!\n\x03Res\x12\x1a\n\x08\x61\x62stract\x18\x01 \x01(\x08R\x08\x61\x62stract\x1a\x37\n\x0bSetAbstract\x1a!\n\x03Req\x12\x1a\n\x08\x61\x62stract\x18\x01 \x01(\x08R\x08\x61\x62stract\x1a\x05\n\x03Res\x1aY\n\tInstances\x1a\x05\n\x03Req\x1a\x45\n\x04Iter\x12\x0e\n\x02id\x18\x01 \x01(\x05R\x02id\x1a-\n\x03Res\x12&\n\x05thing\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\x05thing\x1aj\n\nAttributes\x1a\x05\n\x03Req\x1aU\n\x04Iter\x12\x0e\n\x02id\x18\x01 \x01(\x05R\x02id\x1a=\n\x03Res\x12\x36\n\rattributeType\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\rattributeType\x1a\x64\n\x04Keys\x1a\x05\n\x03Req\x1aU\n\x04Iter\x12\x0e\n\x02id\x18\x01 \x01(\x05R\x02id\x1a=\n\x03Res\x12\x36\n\rattributeType\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\rattributeType\x1aU\n\x07Playing\x1a\x05\n\x03Req\x1a\x43\n\x04Iter\x12\x0e\n\x02id\x18\x01 \x01(\x05R\x02id\x1a+\n\x03Res\x12$\n\x04role\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\x04role\x1aK\n\x03Key\x1a=\n\x03Req\x12\x36\n\rattributeType\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\rattributeType\x1a\x05\n\x03Res\x1aK\n\x03Has\x1a=\n\x03Req\x12\x36\n\rattributeType\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\rattributeType\x1a\x05\n\x03Res\x1a;\n\x05Plays\x1a+\n\x03Req\x12$\n\x04role\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\x04role\x1a\x05\n\x03Res\x1aM\n\x05Unkey\x1a=\n\x03Req\x12\x36\n\rattributeType\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\rattributeType\x1a\x05\n\x03Res\x1aM\n\x05Unhas\x1a=\n\x03Req\x12\x36\n\rattributeType\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\rattributeType\x1a\x05\n\x03Res\x1a<\n\x06Unplay\x1a+\n\x03Req\x12$\n\x04role\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\x04role\x1a\x05\n\x03Res\"N\n\nEntityType\x1a@\n\x06\x43reate\x1a\x05\n\x03Req\x1a/\n\x03Res\x12(\n\x06\x65ntity\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\x06\x65ntity\"\xa8\x02\n\x0cRelationType\x1a\x44\n\x06\x43reate\x1a\x05\n\x03Req\x1a\x33\n\x03Res\x12,\n\x08relation\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\x08relation\x1aS\n\x05Roles\x1a\x05\n\x03Req\x1a\x43\n\x04Iter\x12\x0e\n\x02id\x18\x01 \x01(\x05R\x02id\x1a+\n\x03Res\x12$\n\x04role\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\x04role\x1a=\n\x07Relates\x1a+\n\x03Req\x12$\n\x04role\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\x04role\x1a\x05\n\x03Res\x1a>\n\x08Unrelate\x1a+\n\x03Req\x12$\n\x04role\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\x04role\x1a\x05\n\x03Res\"\xee\x04\n\rAttributeType\x1ar\n\x06\x43reate\x1a\x31\n\x03Req\x12*\n\x05value\x18\x01 \x01(\x0b\x32\x14.session.ValueObjectR\x05value\x1a\x35\n\x03Res\x12.\n\tattribute\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\tattribute\x1a\xa3\x01\n\tAttribute\x1a\x31\n\x03Req\x12*\n\x05value\x18\x01 \x01(\x0b\x32\x14.session.ValueObjectR\x05value\x1a\x63\n\x03Res\x12\x30\n\tattribute\x18\x01 \x01(\x0b\x32\x10.session.ConceptH\x00R\tattribute\x12#\n\x04null\x18\x02 \x01(\x0b\x32\r.session.NullH\x00R\x04nullB\x05\n\x03res\x1a\x84\x01\n\x08\x44\x61taType\x1a\x05\n\x03Req\x1aq\n\x03Res\x12>\n\x08\x64\x61taType\x18\x01 \x01(\x0e\x32 .session.AttributeType.DATA_TYPEH\x00R\x08\x64\x61taType\x12#\n\x04null\x18\x02 \x01(\x0b\x32\r.session.NullH\x00R\x04nullB\x05\n\x03res\x1a.\n\x08GetRegex\x1a\x05\n\x03Req\x1a\x1b\n\x03Res\x12\x14\n\x05regex\x18\x01 \x01(\tR\x05regex\x1a.\n\x08SetRegex\x1a\x1b\n\x03Req\x12\x14\n\x05regex\x18\x01 \x01(\tR\x05regex\x1a\x05\n\x03Res\"\\\n\tDATA_TYPE\x12\n\n\x06STRING\x10\x00\x12\x0b\n\x07\x42OOLEAN\x10\x01\x12\x0b\n\x07INTEGER\x10\x02\x12\x08\n\x04LONG\x10\x03\x12\t\n\x05\x46LOAT\x10\x04\x12\n\n\x06\x44OUBLE\x10\x05\x12\x08\n\x04\x44\x41TE\x10\x06\"\xcf\x06\n\x05Thing\x1a\x36\n\nIsInferred\x1a\x05\n\x03Req\x1a!\n\x03Res\x12\x1a\n\x08inferred\x18\x01 \x01(\x08R\x08inferred\x1a:\n\x04Type\x1a\x05\n\x03Req\x1a+\n\x03Res\x12$\n\x04type\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\x04type\x1a\x96\x01\n\x04Keys\x1a?\n\x03Req\x12\x38\n\x0e\x61ttributeTypes\x18\x01 \x03(\x0b\x32\x10.session.ConceptR\x0e\x61ttributeTypes\x1aM\n\x04Iter\x12\x0e\n\x02id\x18\x01 \x01(\x05R\x02id\x1a\x35\n\x03Res\x12.\n\tattribute\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\tattribute\x1a\x9c\x01\n\nAttributes\x1a?\n\x03Req\x12\x38\n\x0e\x61ttributeTypes\x18\x01 \x03(\x0b\x32\x10.session.ConceptR\x0e\x61ttributeTypes\x1aM\n\x04Iter\x12\x0e\n\x02id\x18\x01 \x01(\x05R\x02id\x1a\x35\n\x03Res\x12.\n\tattribute\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\tattribute\x1a\x87\x01\n\tRelations\x1a-\n\x03Req\x12&\n\x05roles\x18\x01 \x03(\x0b\x32\x10.session.ConceptR\x05roles\x1aK\n\x04Iter\x12\x0e\n\x02id\x18\x01 \x01(\x05R\x02id\x1a\x33\n\x03Res\x12,\n\x08relation\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\x08relation\x1aS\n\x05Roles\x1a\x05\n\x03Req\x1a\x43\n\x04Iter\x12\x0e\n\x02id\x18\x01 \x01(\x05R\x02id\x1a+\n\x03Res\x12$\n\x04role\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\x04role\x1at\n\x06Relhas\x1a\x35\n\x03Req\x12.\n\tattribute\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\tattribute\x1a\x33\n\x03Res\x12,\n\x08relation\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\x08relation\x1a\x45\n\x05Unhas\x1a\x35\n\x03Req\x12.\n\tattribute\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\tattribute\x1a\x05\n\x03Res\"\xeb\x03\n\x08Relation\x1a\x86\x01\n\x0eRolePlayersMap\x1a\x05\n\x03Req\x1am\n\x04Iter\x12\x0e\n\x02id\x18\x01 \x01(\x05R\x02id\x1aU\n\x03Res\x12$\n\x04role\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\x04role\x12(\n\x06player\x18\x02 \x01(\x0b\x32\x10.session.ConceptR\x06player\x1a\x83\x01\n\x0bRolePlayers\x1a-\n\x03Req\x12&\n\x05roles\x18\x01 \x03(\x0b\x32\x10.session.ConceptR\x05roles\x1a\x45\n\x04Iter\x12\x0e\n\x02id\x18\x01 \x01(\x05R\x02id\x1a-\n\x03Res\x12&\n\x05thing\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\x05thing\x1a\x66\n\x06\x41ssign\x1aU\n\x03Req\x12$\n\x04role\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\x04role\x12(\n\x06player\x18\x02 \x01(\x0b\x32\x10.session.ConceptR\x06player\x1a\x05\n\x03Res\x1ah\n\x08Unassign\x1aU\n\x03Req\x12$\n\x04role\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\x04role\x12(\n\x06player\x18\x02 \x01(\x0b\x32\x10.session.ConceptR\x06player\x1a\x05\n\x03Res\"\xa6\x01\n\tAttribute\x1a\x41\n\x05Value\x1a\x05\n\x03Req\x1a\x31\n\x03Res\x12*\n\x05value\x18\x01 \x01(\x0b\x32\x14.session.ValueObjectR\x05value\x1aV\n\x06Owners\x1a\x05\n\x03Req\x1a\x45\n\x04Iter\x12\x0e\n\x02id\x18\x01 \x01(\x05R\x02id\x1a-\n\x03Res\x12&\n\x05thing\x18\x01 \x01(\x0b\x32\x10.session.ConceptR\x05thing\"\xc6\x01\n\x0bValueObject\x12\x18\n\x06string\x18\x01 \x01(\tH\x00R\x06string\x12\x1a\n\x07\x62oolean\x18\x02 \x01(\x08H\x00R\x07\x62oolean\x12\x1a\n\x07integer\x18\x03 \x01(\x05H\x00R\x07integer\x12\x14\n\x04long\x18\x04 \x01(\x03H\x00R\x04long\x12\x16\n\x05\x66loat\x18\x05 \x01(\x02H\x00R\x05\x66loat\x12\x18\n\x06\x64ouble\x18\x06 \x01(\x01H\x00R\x06\x64ouble\x12\x14\n\x04\x64\x61te\x18\x07 \x01(\x03H\x00R\x04\x64\x61teB\x07\n\x05valueB&\n\x16grakn.protocol.sessionB\x0c\x43onceptProtob\x06proto3')
)



_CONCEPT_BASE_TYPE = _descriptor.EnumDescriptor(
  name='BASE_TYPE',
  full_name='session.Concept.BASE_TYPE',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='META_TYPE', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ENTITY_TYPE', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RELATION_TYPE', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ATTRIBUTE_TYPE', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ROLE', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RULE', index=5, number=5,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ENTITY', index=6, number=6,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RELATION', index=7, number=7,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ATTRIBUTE', index=8, number=8,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=9708,
  serialized_end=9851,
)
_sym_db.RegisterEnumDescriptor(_CONCEPT_BASE_TYPE)

_ATTRIBUTETYPE_DATA_TYPE = _descriptor.EnumDescriptor(
  name='DATA_TYPE',
  full_name='session.AttributeType.DATA_TYPE',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='STRING', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='BOOLEAN', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INTEGER', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LONG', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FLOAT', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DOUBLE', index=5, number=5,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DATE', index=6, number=6,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=12686,
  serialized_end=12778,
)
_sym_db.RegisterEnumDescriptor(_ATTRIBUTETYPE_DATA_TYPE)


_METHOD_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Method.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='concept_delete_req', full_name='session.Method.Req.concept_delete_req', index=0,
      number=100, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='conceptDeleteReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='schemaConcept_isImplicit_req', full_name='session.Method.Req.schemaConcept_isImplicit_req', index=1,
      number=200, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='schemaConceptIsImplicitReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='schemaConcept_getLabel_req', full_name='session.Method.Req.schemaConcept_getLabel_req', index=2,
      number=201, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='schemaConceptGetLabelReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='schemaConcept_setLabel_req', full_name='session.Method.Req.schemaConcept_setLabel_req', index=3,
      number=202, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='schemaConceptSetLabelReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='schemaConcept_getSup_req', full_name='session.Method.Req.schemaConcept_getSup_req', index=4,
      number=203, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='schemaConceptGetSupReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='schemaConcept_setSup_req', full_name='session.Method.Req.schemaConcept_setSup_req', index=5,
      number=204, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='schemaConceptSetSupReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='schemaConcept_sups_req', full_name='session.Method.Req.schemaConcept_sups_req', index=6,
      number=205, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='schemaConceptSupsReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='schemaConcept_subs_req', full_name='session.Method.Req.schemaConcept_subs_req', index=7,
      number=206, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='schemaConceptSubsReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='rule_when_req', full_name='session.Method.Req.rule_when_req', index=8,
      number=300, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='ruleWhenReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='rule_then_req', full_name='session.Method.Req.rule_then_req', index=9,
      number=301, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='ruleThenReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='role_relations_req', full_name='session.Method.Req.role_relations_req', index=10,
      number=401, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='roleRelationsReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='role_players_req', full_name='session.Method.Req.role_players_req', index=11,
      number=402, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='rolePlayersReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type_isAbstract_req', full_name='session.Method.Req.type_isAbstract_req', index=12,
      number=500, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='typeIsAbstractReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type_setAbstract_req', full_name='session.Method.Req.type_setAbstract_req', index=13,
      number=501, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='typeSetAbstractReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type_instances_req', full_name='session.Method.Req.type_instances_req', index=14,
      number=502, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='typeInstancesReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type_keys_req', full_name='session.Method.Req.type_keys_req', index=15,
      number=503, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='typeKeysReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type_attributes_req', full_name='session.Method.Req.type_attributes_req', index=16,
      number=504, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='typeAttributesReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type_playing_req', full_name='session.Method.Req.type_playing_req', index=17,
      number=505, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='typePlayingReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type_has_req', full_name='session.Method.Req.type_has_req', index=18,
      number=506, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='typeHasReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type_key_req', full_name='session.Method.Req.type_key_req', index=19,
      number=507, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='typeKeyReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type_plays_req', full_name='session.Method.Req.type_plays_req', index=20,
      number=508, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='typePlaysReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type_unhas_req', full_name='session.Method.Req.type_unhas_req', index=21,
      number=509, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='typeUnhasReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type_unkey_req', full_name='session.Method.Req.type_unkey_req', index=22,
      number=510, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='typeUnkeyReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type_unplay_req', full_name='session.Method.Req.type_unplay_req', index=23,
      number=511, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='typeUnplayReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='entityType_create_req', full_name='session.Method.Req.entityType_create_req', index=24,
      number=600, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='entityTypeCreateReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='relationType_create_req', full_name='session.Method.Req.relationType_create_req', index=25,
      number=700, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='relationTypeCreateReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='relationType_roles_req', full_name='session.Method.Req.relationType_roles_req', index=26,
      number=701, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='relationTypeRolesReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='relationType_relates_req', full_name='session.Method.Req.relationType_relates_req', index=27,
      number=702, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='relationTypeRelatesReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='relationType_unrelate_req', full_name='session.Method.Req.relationType_unrelate_req', index=28,
      number=703, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='relationTypeUnrelateReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='attributeType_create_req', full_name='session.Method.Req.attributeType_create_req', index=29,
      number=800, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='attributeTypeCreateReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='attributeType_attribute_req', full_name='session.Method.Req.attributeType_attribute_req', index=30,
      number=801, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='attributeTypeAttributeReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='attributeType_dataType_req', full_name='session.Method.Req.attributeType_dataType_req', index=31,
      number=802, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='attributeTypeDataTypeReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='attributeType_getRegex_req', full_name='session.Method.Req.attributeType_getRegex_req', index=32,
      number=803, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='attributeTypeGetRegexReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='attributeType_setRegex_req', full_name='session.Method.Req.attributeType_setRegex_req', index=33,
      number=804, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='attributeTypeSetRegexReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='thing_type_req', full_name='session.Method.Req.thing_type_req', index=34,
      number=900, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='thingTypeReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='thing_isInferred_req', full_name='session.Method.Req.thing_isInferred_req', index=35,
      number=901, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='thingIsInferredReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='thing_keys_req', full_name='session.Method.Req.thing_keys_req', index=36,
      number=902, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='thingKeysReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='thing_attributes_req', full_name='session.Method.Req.thing_attributes_req', index=37,
      number=903, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='thingAttributesReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='thing_relations_req', full_name='session.Method.Req.thing_relations_req', index=38,
      number=904, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='thingRelationsReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='thing_roles_req', full_name='session.Method.Req.thing_roles_req', index=39,
      number=905, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='thingRolesReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='thing_relhas_req', full_name='session.Method.Req.thing_relhas_req', index=40,
      number=906, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='thingRelhasReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='thing_unhas_req', full_name='session.Method.Req.thing_unhas_req', index=41,
      number=907, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='thingUnhasReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='relation_rolePlayersMap_req', full_name='session.Method.Req.relation_rolePlayersMap_req', index=42,
      number=1000, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='relationRolePlayersMapReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='relation_rolePlayers_req', full_name='session.Method.Req.relation_rolePlayers_req', index=43,
      number=1001, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='relationRolePlayersReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='relation_assign_req', full_name='session.Method.Req.relation_assign_req', index=44,
      number=1002, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='relationAssignReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='relation_unassign_req', full_name='session.Method.Req.relation_unassign_req', index=45,
      number=1003, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='relationUnassignReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='attribute_value_req', full_name='session.Method.Req.attribute_value_req', index=46,
      number=1100, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='attributeValueReq', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='attribute_owners_req', full_name='session.Method.Req.attribute_owners_req', index=47,
      number=1101, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='attributeOwnersReq', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='req', full_name='session.Method.Req.req',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=46,
  serialized_end=4038,
)

_METHOD_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Method.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='concept_delete_res', full_name='session.Method.Res.concept_delete_res', index=0,
      number=100, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='conceptDeleteRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='schemaConcept_isImplicit_res', full_name='session.Method.Res.schemaConcept_isImplicit_res', index=1,
      number=200, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='schemaConceptIsImplicitRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='schemaConcept_getLabel_res', full_name='session.Method.Res.schemaConcept_getLabel_res', index=2,
      number=201, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='schemaConceptGetLabelRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='schemaConcept_setLabel_res', full_name='session.Method.Res.schemaConcept_setLabel_res', index=3,
      number=202, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='schemaConceptSetLabelRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='schemaConcept_getSup_res', full_name='session.Method.Res.schemaConcept_getSup_res', index=4,
      number=203, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='schemaConceptGetSupRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='schemaConcept_setSup_res', full_name='session.Method.Res.schemaConcept_setSup_res', index=5,
      number=204, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='schemaConceptSetSupRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='schemaConcept_sups_iter', full_name='session.Method.Res.schemaConcept_sups_iter', index=6,
      number=205, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='schemaConceptSupsIter', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='schemaConcept_subs_iter', full_name='session.Method.Res.schemaConcept_subs_iter', index=7,
      number=206, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='schemaConceptSubsIter', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='rule_when_res', full_name='session.Method.Res.rule_when_res', index=8,
      number=300, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='ruleWhenRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='rule_then_res', full_name='session.Method.Res.rule_then_res', index=9,
      number=301, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='ruleThenRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='role_relations_iter', full_name='session.Method.Res.role_relations_iter', index=10,
      number=401, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='roleRelationsIter', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='role_players_iter', full_name='session.Method.Res.role_players_iter', index=11,
      number=402, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='rolePlayersIter', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type_isAbstract_res', full_name='session.Method.Res.type_isAbstract_res', index=12,
      number=500, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='typeIsAbstractRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type_setAbstract_res', full_name='session.Method.Res.type_setAbstract_res', index=13,
      number=501, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='typeSetAbstractRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type_instances_iter', full_name='session.Method.Res.type_instances_iter', index=14,
      number=502, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='typeInstancesIter', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type_keys_iter', full_name='session.Method.Res.type_keys_iter', index=15,
      number=503, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='typeKeysIter', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type_attributes_iter', full_name='session.Method.Res.type_attributes_iter', index=16,
      number=504, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='typeAttributesIter', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type_playing_iter', full_name='session.Method.Res.type_playing_iter', index=17,
      number=505, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='typePlayingIter', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type_has_res', full_name='session.Method.Res.type_has_res', index=18,
      number=506, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='typeHasRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type_key_res', full_name='session.Method.Res.type_key_res', index=19,
      number=507, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='typeKeyRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type_plays_res', full_name='session.Method.Res.type_plays_res', index=20,
      number=508, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='typePlaysRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type_unhas_res', full_name='session.Method.Res.type_unhas_res', index=21,
      number=509, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='typeUnhasRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type_unkey_res', full_name='session.Method.Res.type_unkey_res', index=22,
      number=510, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='typeUnkeyRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type_unplay_res', full_name='session.Method.Res.type_unplay_res', index=23,
      number=511, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='typeUnplayRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='entityType_create_res', full_name='session.Method.Res.entityType_create_res', index=24,
      number=600, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='entityTypeCreateRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='relationType_create_res', full_name='session.Method.Res.relationType_create_res', index=25,
      number=700, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='relationTypeCreateRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='relationType_roles_iter', full_name='session.Method.Res.relationType_roles_iter', index=26,
      number=701, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='relationTypeRolesIter', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='relationType_relates_res', full_name='session.Method.Res.relationType_relates_res', index=27,
      number=702, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='relationTypeRelatesRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='relationType_unrelate_res', full_name='session.Method.Res.relationType_unrelate_res', index=28,
      number=703, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='relationTypeUnrelateRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='attributeType_create_res', full_name='session.Method.Res.attributeType_create_res', index=29,
      number=800, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='attributeTypeCreateRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='attributeType_attribute_res', full_name='session.Method.Res.attributeType_attribute_res', index=30,
      number=801, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='attributeTypeAttributeRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='attributeType_dataType_res', full_name='session.Method.Res.attributeType_dataType_res', index=31,
      number=802, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='attributeTypeDataTypeRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='attributeType_getRegex_res', full_name='session.Method.Res.attributeType_getRegex_res', index=32,
      number=803, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='attributeTypeGetRegexRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='attributeType_setRegex_res', full_name='session.Method.Res.attributeType_setRegex_res', index=33,
      number=804, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='attributeTypeSetRegexRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='thing_type_res', full_name='session.Method.Res.thing_type_res', index=34,
      number=900, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='thingTypeRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='thing_isInferred_res', full_name='session.Method.Res.thing_isInferred_res', index=35,
      number=901, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='thingIsInferredRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='thing_keys_iter', full_name='session.Method.Res.thing_keys_iter', index=36,
      number=902, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='thingKeysIter', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='thing_attributes_iter', full_name='session.Method.Res.thing_attributes_iter', index=37,
      number=903, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='thingAttributesIter', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='thing_relations_iter', full_name='session.Method.Res.thing_relations_iter', index=38,
      number=904, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='thingRelationsIter', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='thing_roles_iter', full_name='session.Method.Res.thing_roles_iter', index=39,
      number=905, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='thingRolesIter', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='thing_relhas_res', full_name='session.Method.Res.thing_relhas_res', index=40,
      number=906, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='thingRelhasRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='thing_unhas_res', full_name='session.Method.Res.thing_unhas_res', index=41,
      number=907, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='thingUnhasRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='relation_rolePlayersMap_iter', full_name='session.Method.Res.relation_rolePlayersMap_iter', index=42,
      number=1000, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='relationRolePlayersMapIter', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='relation_rolePlayers_iter', full_name='session.Method.Res.relation_rolePlayers_iter', index=43,
      number=1001, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='relationRolePlayersIter', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='relation_assign_res', full_name='session.Method.Res.relation_assign_res', index=44,
      number=1002, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='relationAssignRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='relation_unassign_res', full_name='session.Method.Res.relation_unassign_res', index=45,
      number=1003, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='relationUnassignRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='attribute_value_res', full_name='session.Method.Res.attribute_value_res', index=46,
      number=1100, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='attributeValueRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='attribute_owners_iter', full_name='session.Method.Res.attribute_owners_iter', index=47,
      number=1101, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='attributeOwnersIter', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='res', full_name='session.Method.Res.res',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=4041,
  serialized_end=8081,
)

_METHOD_ITER_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Method.Iter.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='schemaConcept_sups_iter_res', full_name='session.Method.Iter.Res.schemaConcept_sups_iter_res', index=0,
      number=205, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='schemaConceptSupsIterRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='schemaConcept_subs_iter_res', full_name='session.Method.Iter.Res.schemaConcept_subs_iter_res', index=1,
      number=206, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='schemaConceptSubsIterRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='role_relations_iter_res', full_name='session.Method.Iter.Res.role_relations_iter_res', index=2,
      number=401, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='roleRelationsIterRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='role_players_iter_res', full_name='session.Method.Iter.Res.role_players_iter_res', index=3,
      number=402, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='rolePlayersIterRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type_instances_iter_res', full_name='session.Method.Iter.Res.type_instances_iter_res', index=4,
      number=502, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='typeInstancesIterRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type_keys_iter_res', full_name='session.Method.Iter.Res.type_keys_iter_res', index=5,
      number=503, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='typeKeysIterRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type_attributes_iter_res', full_name='session.Method.Iter.Res.type_attributes_iter_res', index=6,
      number=504, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='typeAttributesIterRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type_playing_iter_res', full_name='session.Method.Iter.Res.type_playing_iter_res', index=7,
      number=505, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='typePlayingIterRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='relationType_roles_iter_res', full_name='session.Method.Iter.Res.relationType_roles_iter_res', index=8,
      number=701, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='relationTypeRolesIterRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='thing_keys_iter_res', full_name='session.Method.Iter.Res.thing_keys_iter_res', index=9,
      number=902, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='thingKeysIterRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='thing_attributes_iter_res', full_name='session.Method.Iter.Res.thing_attributes_iter_res', index=10,
      number=903, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='thingAttributesIterRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='thing_relations_iter_res', full_name='session.Method.Iter.Res.thing_relations_iter_res', index=11,
      number=904, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='thingRelationsIterRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='thing_roles_iter_res', full_name='session.Method.Iter.Res.thing_roles_iter_res', index=12,
      number=905, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='thingRolesIterRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='relation_rolePlayersMap_iter_res', full_name='session.Method.Iter.Res.relation_rolePlayersMap_iter_res', index=13,
      number=1000, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='relationRolePlayersMapIterRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='relation_rolePlayers_iter_res', full_name='session.Method.Iter.Res.relation_rolePlayers_iter_res', index=14,
      number=1001, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='relationRolePlayersIterRes', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='attribute_owners_iter_res', full_name='session.Method.Iter.Res.attribute_owners_iter_res', index=15,
      number=1101, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='attributeOwnersIterRes', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8093,
  serialized_end=9589,
)

_METHOD_ITER = _descriptor.Descriptor(
  name='Iter',
  full_name='session.Method.Iter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_METHOD_ITER_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8084,
  serialized_end=9589,
)

_METHOD = _descriptor.Descriptor(
  name='Method',
  full_name='session.Method',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_METHOD_REQ, _METHOD_RES, _METHOD_ITER, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=35,
  serialized_end=9589,
)


_NULL = _descriptor.Descriptor(
  name='Null',
  full_name='session.Null',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9591,
  serialized_end=9597,
)


_CONCEPT_DELETE_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Concept.Delete.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=51,
)

_CONCEPT_DELETE_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Concept.Delete.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4041,
  serialized_end=4046,
)

_CONCEPT_DELETE = _descriptor.Descriptor(
  name='Delete',
  full_name='session.Concept.Delete',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_CONCEPT_DELETE_REQ, _CONCEPT_DELETE_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9683,
  serialized_end=9705,
)

_CONCEPT = _descriptor.Descriptor(
  name='Concept',
  full_name='session.Concept',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='session.Concept.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='id', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='baseType', full_name='session.Concept.baseType', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='baseType', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_CONCEPT_DELETE, ],
  enum_types=[
    _CONCEPT_BASE_TYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9600,
  serialized_end=9851,
)


_SCHEMACONCEPT_GETLABEL_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.SchemaConcept.GetLabel.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=51,
)

_SCHEMACONCEPT_GETLABEL_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.SchemaConcept.GetLabel.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='label', full_name='session.SchemaConcept.GetLabel.Res.label', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='label', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9890,
  serialized_end=9917,
)

_SCHEMACONCEPT_GETLABEL = _descriptor.Descriptor(
  name='GetLabel',
  full_name='session.SchemaConcept.GetLabel',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_SCHEMACONCEPT_GETLABEL_REQ, _SCHEMACONCEPT_GETLABEL_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9871,
  serialized_end=9917,
)

_SCHEMACONCEPT_SETLABEL_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.SchemaConcept.SetLabel.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='label', full_name='session.SchemaConcept.SetLabel.Req.label', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='label', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9931,
  serialized_end=9958,
)

_SCHEMACONCEPT_SETLABEL_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.SchemaConcept.SetLabel.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4041,
  serialized_end=4046,
)

_SCHEMACONCEPT_SETLABEL = _descriptor.Descriptor(
  name='SetLabel',
  full_name='session.SchemaConcept.SetLabel',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_SCHEMACONCEPT_SETLABEL_REQ, _SCHEMACONCEPT_SETLABEL_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9919,
  serialized_end=9965,
)

_SCHEMACONCEPT_ISIMPLICIT_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.SchemaConcept.IsImplicit.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=51,
)

_SCHEMACONCEPT_ISIMPLICIT_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.SchemaConcept.IsImplicit.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='implicit', full_name='session.SchemaConcept.IsImplicit.Res.implicit', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='implicit', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9988,
  serialized_end=10021,
)

_SCHEMACONCEPT_ISIMPLICIT = _descriptor.Descriptor(
  name='IsImplicit',
  full_name='session.SchemaConcept.IsImplicit',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_SCHEMACONCEPT_ISIMPLICIT_REQ, _SCHEMACONCEPT_ISIMPLICIT_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9967,
  serialized_end=10021,
)

_SCHEMACONCEPT_GETSUP_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.SchemaConcept.GetSup.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=51,
)

_SCHEMACONCEPT_GETSUP_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.SchemaConcept.GetSup.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='schemaConcept', full_name='session.SchemaConcept.GetSup.Res.schemaConcept', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='schemaConcept', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='null', full_name='session.SchemaConcept.GetSup.Res.null', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='null', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='res', full_name='session.SchemaConcept.GetSup.Res.res',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=10040,
  serialized_end=10147,
)

_SCHEMACONCEPT_GETSUP = _descriptor.Descriptor(
  name='GetSup',
  full_name='session.SchemaConcept.GetSup',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_SCHEMACONCEPT_GETSUP_REQ, _SCHEMACONCEPT_GETSUP_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10023,
  serialized_end=10147,
)

_SCHEMACONCEPT_SETSUP_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.SchemaConcept.SetSup.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='schemaConcept', full_name='session.SchemaConcept.SetSup.Req.schemaConcept', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='schemaConcept', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10159,
  serialized_end=10220,
)

_SCHEMACONCEPT_SETSUP_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.SchemaConcept.SetSup.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4041,
  serialized_end=4046,
)

_SCHEMACONCEPT_SETSUP = _descriptor.Descriptor(
  name='SetSup',
  full_name='session.SchemaConcept.SetSup',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_SCHEMACONCEPT_SETSUP_REQ, _SCHEMACONCEPT_SETSUP_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10149,
  serialized_end=10227,
)

_SCHEMACONCEPT_SUPS_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.SchemaConcept.Sups.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=51,
)

_SCHEMACONCEPT_SUPS_ITER_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.SchemaConcept.Sups.Iter.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='schemaConcept', full_name='session.SchemaConcept.Sups.Iter.Res.schemaConcept', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='schemaConcept', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10268,
  serialized_end=10329,
)

_SCHEMACONCEPT_SUPS_ITER = _descriptor.Descriptor(
  name='Iter',
  full_name='session.SchemaConcept.Sups.Iter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='session.SchemaConcept.Sups.Iter.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='id', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_SCHEMACONCEPT_SUPS_ITER_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10244,
  serialized_end=10329,
)

_SCHEMACONCEPT_SUPS = _descriptor.Descriptor(
  name='Sups',
  full_name='session.SchemaConcept.Sups',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_SCHEMACONCEPT_SUPS_REQ, _SCHEMACONCEPT_SUPS_ITER, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10229,
  serialized_end=10329,
)

_SCHEMACONCEPT_SUBS_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.SchemaConcept.Subs.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=51,
)

_SCHEMACONCEPT_SUBS_ITER_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.SchemaConcept.Subs.Iter.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='schemaConcept', full_name='session.SchemaConcept.Subs.Iter.Res.schemaConcept', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='schemaConcept', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10268,
  serialized_end=10329,
)

_SCHEMACONCEPT_SUBS_ITER = _descriptor.Descriptor(
  name='Iter',
  full_name='session.SchemaConcept.Subs.Iter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='session.SchemaConcept.Subs.Iter.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='id', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_SCHEMACONCEPT_SUBS_ITER_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10244,
  serialized_end=10329,
)

_SCHEMACONCEPT_SUBS = _descriptor.Descriptor(
  name='Subs',
  full_name='session.SchemaConcept.Subs',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_SCHEMACONCEPT_SUBS_REQ, _SCHEMACONCEPT_SUBS_ITER, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10331,
  serialized_end=10431,
)

_SCHEMACONCEPT = _descriptor.Descriptor(
  name='SchemaConcept',
  full_name='session.SchemaConcept',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_SCHEMACONCEPT_GETLABEL, _SCHEMACONCEPT_SETLABEL, _SCHEMACONCEPT_ISIMPLICIT, _SCHEMACONCEPT_GETSUP, _SCHEMACONCEPT_SETSUP, _SCHEMACONCEPT_SUPS, _SCHEMACONCEPT_SUBS, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9854,
  serialized_end=10431,
)


_RULE_WHEN_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Rule.When.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=51,
)

_RULE_WHEN_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Rule.When.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='pattern', full_name='session.Rule.When.Res.pattern', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='pattern', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='null', full_name='session.Rule.When.Res.null', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='null', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='res', full_name='session.Rule.When.Res.res',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=10457,
  serialized_end=10534,
)

_RULE_WHEN = _descriptor.Descriptor(
  name='When',
  full_name='session.Rule.When',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_RULE_WHEN_REQ, _RULE_WHEN_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10442,
  serialized_end=10534,
)

_RULE_THEN_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Rule.Then.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=51,
)

_RULE_THEN_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Rule.Then.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='pattern', full_name='session.Rule.Then.Res.pattern', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='pattern', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='null', full_name='session.Rule.Then.Res.null', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='null', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='res', full_name='session.Rule.Then.Res.res',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=10457,
  serialized_end=10534,
)

_RULE_THEN = _descriptor.Descriptor(
  name='Then',
  full_name='session.Rule.Then',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_RULE_THEN_REQ, _RULE_THEN_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10536,
  serialized_end=10628,
)

_RULE = _descriptor.Descriptor(
  name='Rule',
  full_name='session.Rule',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_RULE_WHEN, _RULE_THEN, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10434,
  serialized_end=10628,
)


_ROLE_RELATIONS_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Role.Relations.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=51,
)

_ROLE_RELATIONS_ITER_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Role.Relations.Iter.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='relationType', full_name='session.Role.Relations.Iter.Res.relationType', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='relationType', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10683,
  serialized_end=10742,
)

_ROLE_RELATIONS_ITER = _descriptor.Descriptor(
  name='Iter',
  full_name='session.Role.Relations.Iter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='session.Role.Relations.Iter.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='id', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_ROLE_RELATIONS_ITER_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10659,
  serialized_end=10742,
)

_ROLE_RELATIONS = _descriptor.Descriptor(
  name='Relations',
  full_name='session.Role.Relations',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_ROLE_RELATIONS_REQ, _ROLE_RELATIONS_ITER, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10639,
  serialized_end=10742,
)

_ROLE_PLAYERS_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Role.Players.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=51,
)

_ROLE_PLAYERS_ITER_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Role.Players.Iter.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='session.Role.Players.Iter.Res.type', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='type', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10786,
  serialized_end=10829,
)

_ROLE_PLAYERS_ITER = _descriptor.Descriptor(
  name='Iter',
  full_name='session.Role.Players.Iter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='session.Role.Players.Iter.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='id', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_ROLE_PLAYERS_ITER_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10762,
  serialized_end=10829,
)

_ROLE_PLAYERS = _descriptor.Descriptor(
  name='Players',
  full_name='session.Role.Players',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_ROLE_PLAYERS_REQ, _ROLE_PLAYERS_ITER, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10744,
  serialized_end=10829,
)

_ROLE = _descriptor.Descriptor(
  name='Role',
  full_name='session.Role',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_ROLE_RELATIONS, _ROLE_PLAYERS, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10631,
  serialized_end=10829,
)


_TYPE_ISABSTRACT_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Type.IsAbstract.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=51,
)

_TYPE_ISABSTRACT_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Type.IsAbstract.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='abstract', full_name='session.Type.IsAbstract.Res.abstract', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='abstract', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10861,
  serialized_end=10894,
)

_TYPE_ISABSTRACT = _descriptor.Descriptor(
  name='IsAbstract',
  full_name='session.Type.IsAbstract',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_TYPE_ISABSTRACT_REQ, _TYPE_ISABSTRACT_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10840,
  serialized_end=10894,
)

_TYPE_SETABSTRACT_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Type.SetAbstract.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='abstract', full_name='session.Type.SetAbstract.Req.abstract', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='abstract', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10911,
  serialized_end=10944,
)

_TYPE_SETABSTRACT_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Type.SetAbstract.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4041,
  serialized_end=4046,
)

_TYPE_SETABSTRACT = _descriptor.Descriptor(
  name='SetAbstract',
  full_name='session.Type.SetAbstract',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_TYPE_SETABSTRACT_REQ, _TYPE_SETABSTRACT_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10896,
  serialized_end=10951,
)

_TYPE_INSTANCES_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Type.Instances.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=51,
)

_TYPE_INSTANCES_ITER_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Type.Instances.Iter.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='thing', full_name='session.Type.Instances.Iter.Res.thing', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='thing', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10997,
  serialized_end=11042,
)

_TYPE_INSTANCES_ITER = _descriptor.Descriptor(
  name='Iter',
  full_name='session.Type.Instances.Iter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='session.Type.Instances.Iter.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='id', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_TYPE_INSTANCES_ITER_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10973,
  serialized_end=11042,
)

_TYPE_INSTANCES = _descriptor.Descriptor(
  name='Instances',
  full_name='session.Type.Instances',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_TYPE_INSTANCES_REQ, _TYPE_INSTANCES_ITER, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10953,
  serialized_end=11042,
)

_TYPE_ATTRIBUTES_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Type.Attributes.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=51,
)

_TYPE_ATTRIBUTES_ITER_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Type.Attributes.Iter.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='attributeType', full_name='session.Type.Attributes.Iter.Res.attributeType', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='attributeType', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11089,
  serialized_end=11150,
)

_TYPE_ATTRIBUTES_ITER = _descriptor.Descriptor(
  name='Iter',
  full_name='session.Type.Attributes.Iter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='session.Type.Attributes.Iter.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='id', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_TYPE_ATTRIBUTES_ITER_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11065,
  serialized_end=11150,
)

_TYPE_ATTRIBUTES = _descriptor.Descriptor(
  name='Attributes',
  full_name='session.Type.Attributes',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_TYPE_ATTRIBUTES_REQ, _TYPE_ATTRIBUTES_ITER, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11044,
  serialized_end=11150,
)

_TYPE_KEYS_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Type.Keys.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=51,
)

_TYPE_KEYS_ITER_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Type.Keys.Iter.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='attributeType', full_name='session.Type.Keys.Iter.Res.attributeType', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='attributeType', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11089,
  serialized_end=11150,
)

_TYPE_KEYS_ITER = _descriptor.Descriptor(
  name='Iter',
  full_name='session.Type.Keys.Iter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='session.Type.Keys.Iter.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='id', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_TYPE_KEYS_ITER_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11065,
  serialized_end=11150,
)

_TYPE_KEYS = _descriptor.Descriptor(
  name='Keys',
  full_name='session.Type.Keys',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_TYPE_KEYS_REQ, _TYPE_KEYS_ITER, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11152,
  serialized_end=11252,
)

_TYPE_PLAYING_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Type.Playing.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=51,
)

_TYPE_PLAYING_ITER_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Type.Playing.Iter.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='role', full_name='session.Type.Playing.Iter.Res.role', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='role', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11296,
  serialized_end=11339,
)

_TYPE_PLAYING_ITER = _descriptor.Descriptor(
  name='Iter',
  full_name='session.Type.Playing.Iter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='session.Type.Playing.Iter.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='id', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_TYPE_PLAYING_ITER_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11272,
  serialized_end=11339,
)

_TYPE_PLAYING = _descriptor.Descriptor(
  name='Playing',
  full_name='session.Type.Playing',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_TYPE_PLAYING_REQ, _TYPE_PLAYING_ITER, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11254,
  serialized_end=11339,
)

_TYPE_KEY_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Type.Key.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='attributeType', full_name='session.Type.Key.Req.attributeType', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='attributeType', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11348,
  serialized_end=11409,
)

_TYPE_KEY_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Type.Key.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4041,
  serialized_end=4046,
)

_TYPE_KEY = _descriptor.Descriptor(
  name='Key',
  full_name='session.Type.Key',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_TYPE_KEY_REQ, _TYPE_KEY_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11341,
  serialized_end=11416,
)

_TYPE_HAS_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Type.Has.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='attributeType', full_name='session.Type.Has.Req.attributeType', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='attributeType', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11348,
  serialized_end=11409,
)

_TYPE_HAS_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Type.Has.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4041,
  serialized_end=4046,
)

_TYPE_HAS = _descriptor.Descriptor(
  name='Has',
  full_name='session.Type.Has',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_TYPE_HAS_REQ, _TYPE_HAS_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11418,
  serialized_end=11493,
)

_TYPE_PLAYS_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Type.Plays.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='role', full_name='session.Type.Plays.Req.role', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='role', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11504,
  serialized_end=11547,
)

_TYPE_PLAYS_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Type.Plays.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4041,
  serialized_end=4046,
)

_TYPE_PLAYS = _descriptor.Descriptor(
  name='Plays',
  full_name='session.Type.Plays',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_TYPE_PLAYS_REQ, _TYPE_PLAYS_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11495,
  serialized_end=11554,
)

_TYPE_UNKEY_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Type.Unkey.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='attributeType', full_name='session.Type.Unkey.Req.attributeType', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='attributeType', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11348,
  serialized_end=11409,
)

_TYPE_UNKEY_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Type.Unkey.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4041,
  serialized_end=4046,
)

_TYPE_UNKEY = _descriptor.Descriptor(
  name='Unkey',
  full_name='session.Type.Unkey',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_TYPE_UNKEY_REQ, _TYPE_UNKEY_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11556,
  serialized_end=11633,
)

_TYPE_UNHAS_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Type.Unhas.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='attributeType', full_name='session.Type.Unhas.Req.attributeType', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='attributeType', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11348,
  serialized_end=11409,
)

_TYPE_UNHAS_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Type.Unhas.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4041,
  serialized_end=4046,
)

_TYPE_UNHAS = _descriptor.Descriptor(
  name='Unhas',
  full_name='session.Type.Unhas',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_TYPE_UNHAS_REQ, _TYPE_UNHAS_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11635,
  serialized_end=11712,
)

_TYPE_UNPLAY_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Type.Unplay.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='role', full_name='session.Type.Unplay.Req.role', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='role', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11504,
  serialized_end=11547,
)

_TYPE_UNPLAY_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Type.Unplay.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4041,
  serialized_end=4046,
)

_TYPE_UNPLAY = _descriptor.Descriptor(
  name='Unplay',
  full_name='session.Type.Unplay',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_TYPE_UNPLAY_REQ, _TYPE_UNPLAY_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11714,
  serialized_end=11774,
)

_TYPE = _descriptor.Descriptor(
  name='Type',
  full_name='session.Type',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_TYPE_ISABSTRACT, _TYPE_SETABSTRACT, _TYPE_INSTANCES, _TYPE_ATTRIBUTES, _TYPE_KEYS, _TYPE_PLAYING, _TYPE_KEY, _TYPE_HAS, _TYPE_PLAYS, _TYPE_UNKEY, _TYPE_UNHAS, _TYPE_UNPLAY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10832,
  serialized_end=11774,
)


_ENTITYTYPE_CREATE_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.EntityType.Create.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=51,
)

_ENTITYTYPE_CREATE_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.EntityType.Create.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='entity', full_name='session.EntityType.Create.Res.entity', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='entity', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11807,
  serialized_end=11854,
)

_ENTITYTYPE_CREATE = _descriptor.Descriptor(
  name='Create',
  full_name='session.EntityType.Create',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_ENTITYTYPE_CREATE_REQ, _ENTITYTYPE_CREATE_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11790,
  serialized_end=11854,
)

_ENTITYTYPE = _descriptor.Descriptor(
  name='EntityType',
  full_name='session.EntityType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_ENTITYTYPE_CREATE, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11776,
  serialized_end=11854,
)


_RELATIONTYPE_CREATE_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.RelationType.Create.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=51,
)

_RELATIONTYPE_CREATE_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.RelationType.Create.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='relation', full_name='session.RelationType.Create.Res.relation', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='relation', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11890,
  serialized_end=11941,
)

_RELATIONTYPE_CREATE = _descriptor.Descriptor(
  name='Create',
  full_name='session.RelationType.Create',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_RELATIONTYPE_CREATE_REQ, _RELATIONTYPE_CREATE_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11873,
  serialized_end=11941,
)

_RELATIONTYPE_ROLES_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.RelationType.Roles.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=51,
)

_RELATIONTYPE_ROLES_ITER_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.RelationType.Roles.Iter.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='role', full_name='session.RelationType.Roles.Iter.Res.role', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='role', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11296,
  serialized_end=11339,
)

_RELATIONTYPE_ROLES_ITER = _descriptor.Descriptor(
  name='Iter',
  full_name='session.RelationType.Roles.Iter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='session.RelationType.Roles.Iter.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='id', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_RELATIONTYPE_ROLES_ITER_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11272,
  serialized_end=11339,
)

_RELATIONTYPE_ROLES = _descriptor.Descriptor(
  name='Roles',
  full_name='session.RelationType.Roles',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_RELATIONTYPE_ROLES_REQ, _RELATIONTYPE_ROLES_ITER, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11943,
  serialized_end=12026,
)

_RELATIONTYPE_RELATES_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.RelationType.Relates.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='role', full_name='session.RelationType.Relates.Req.role', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='role', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11504,
  serialized_end=11547,
)

_RELATIONTYPE_RELATES_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.RelationType.Relates.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4041,
  serialized_end=4046,
)

_RELATIONTYPE_RELATES = _descriptor.Descriptor(
  name='Relates',
  full_name='session.RelationType.Relates',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_RELATIONTYPE_RELATES_REQ, _RELATIONTYPE_RELATES_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12028,
  serialized_end=12089,
)

_RELATIONTYPE_UNRELATE_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.RelationType.Unrelate.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='role', full_name='session.RelationType.Unrelate.Req.role', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='role', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11504,
  serialized_end=11547,
)

_RELATIONTYPE_UNRELATE_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.RelationType.Unrelate.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4041,
  serialized_end=4046,
)

_RELATIONTYPE_UNRELATE = _descriptor.Descriptor(
  name='Unrelate',
  full_name='session.RelationType.Unrelate',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_RELATIONTYPE_UNRELATE_REQ, _RELATIONTYPE_UNRELATE_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12091,
  serialized_end=12153,
)

_RELATIONTYPE = _descriptor.Descriptor(
  name='RelationType',
  full_name='session.RelationType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_RELATIONTYPE_CREATE, _RELATIONTYPE_ROLES, _RELATIONTYPE_RELATES, _RELATIONTYPE_UNRELATE, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11857,
  serialized_end=12153,
)


_ATTRIBUTETYPE_CREATE_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.AttributeType.Create.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='session.AttributeType.Create.Req.value', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='value', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12183,
  serialized_end=12232,
)

_ATTRIBUTETYPE_CREATE_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.AttributeType.Create.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='attribute', full_name='session.AttributeType.Create.Res.attribute', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='attribute', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12234,
  serialized_end=12287,
)

_ATTRIBUTETYPE_CREATE = _descriptor.Descriptor(
  name='Create',
  full_name='session.AttributeType.Create',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_ATTRIBUTETYPE_CREATE_REQ, _ATTRIBUTETYPE_CREATE_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12173,
  serialized_end=12287,
)

_ATTRIBUTETYPE_ATTRIBUTE_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.AttributeType.Attribute.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='session.AttributeType.Attribute.Req.value', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='value', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12183,
  serialized_end=12232,
)

_ATTRIBUTETYPE_ATTRIBUTE_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.AttributeType.Attribute.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='attribute', full_name='session.AttributeType.Attribute.Res.attribute', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='attribute', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='null', full_name='session.AttributeType.Attribute.Res.null', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='null', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='res', full_name='session.AttributeType.Attribute.Res.res',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=12354,
  serialized_end=12453,
)

_ATTRIBUTETYPE_ATTRIBUTE = _descriptor.Descriptor(
  name='Attribute',
  full_name='session.AttributeType.Attribute',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_ATTRIBUTETYPE_ATTRIBUTE_REQ, _ATTRIBUTETYPE_ATTRIBUTE_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12290,
  serialized_end=12453,
)

_ATTRIBUTETYPE_DATATYPE_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.AttributeType.DataType.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=51,
)

_ATTRIBUTETYPE_DATATYPE_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.AttributeType.DataType.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='dataType', full_name='session.AttributeType.DataType.Res.dataType', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='dataType', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='null', full_name='session.AttributeType.DataType.Res.null', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='null', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='res', full_name='session.AttributeType.DataType.Res.res',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=12475,
  serialized_end=12588,
)

_ATTRIBUTETYPE_DATATYPE = _descriptor.Descriptor(
  name='DataType',
  full_name='session.AttributeType.DataType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_ATTRIBUTETYPE_DATATYPE_REQ, _ATTRIBUTETYPE_DATATYPE_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12456,
  serialized_end=12588,
)

_ATTRIBUTETYPE_GETREGEX_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.AttributeType.GetRegex.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=51,
)

_ATTRIBUTETYPE_GETREGEX_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.AttributeType.GetRegex.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='regex', full_name='session.AttributeType.GetRegex.Res.regex', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='regex', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12609,
  serialized_end=12636,
)

_ATTRIBUTETYPE_GETREGEX = _descriptor.Descriptor(
  name='GetRegex',
  full_name='session.AttributeType.GetRegex',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_ATTRIBUTETYPE_GETREGEX_REQ, _ATTRIBUTETYPE_GETREGEX_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12590,
  serialized_end=12636,
)

_ATTRIBUTETYPE_SETREGEX_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.AttributeType.SetRegex.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='regex', full_name='session.AttributeType.SetRegex.Req.regex', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='regex', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12650,
  serialized_end=12677,
)

_ATTRIBUTETYPE_SETREGEX_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.AttributeType.SetRegex.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4041,
  serialized_end=4046,
)

_ATTRIBUTETYPE_SETREGEX = _descriptor.Descriptor(
  name='SetRegex',
  full_name='session.AttributeType.SetRegex',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_ATTRIBUTETYPE_SETREGEX_REQ, _ATTRIBUTETYPE_SETREGEX_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12638,
  serialized_end=12684,
)

_ATTRIBUTETYPE = _descriptor.Descriptor(
  name='AttributeType',
  full_name='session.AttributeType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_ATTRIBUTETYPE_CREATE, _ATTRIBUTETYPE_ATTRIBUTE, _ATTRIBUTETYPE_DATATYPE, _ATTRIBUTETYPE_GETREGEX, _ATTRIBUTETYPE_SETREGEX, ],
  enum_types=[
    _ATTRIBUTETYPE_DATA_TYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12156,
  serialized_end=12778,
)


_THING_ISINFERRED_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Thing.IsInferred.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=51,
)

_THING_ISINFERRED_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Thing.IsInferred.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='inferred', full_name='session.Thing.IsInferred.Res.inferred', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='inferred', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12811,
  serialized_end=12844,
)

_THING_ISINFERRED = _descriptor.Descriptor(
  name='IsInferred',
  full_name='session.Thing.IsInferred',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_THING_ISINFERRED_REQ, _THING_ISINFERRED_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12790,
  serialized_end=12844,
)

_THING_TYPE_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Thing.Type.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=51,
)

_THING_TYPE_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Thing.Type.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='session.Thing.Type.Res.type', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='type', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10786,
  serialized_end=10829,
)

_THING_TYPE = _descriptor.Descriptor(
  name='Type',
  full_name='session.Thing.Type',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_THING_TYPE_REQ, _THING_TYPE_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12846,
  serialized_end=12904,
)

_THING_KEYS_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Thing.Keys.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='attributeTypes', full_name='session.Thing.Keys.Req.attributeTypes', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='attributeTypes', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12915,
  serialized_end=12978,
)

_THING_KEYS_ITER_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Thing.Keys.Iter.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='attribute', full_name='session.Thing.Keys.Iter.Res.attribute', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='attribute', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12234,
  serialized_end=12287,
)

_THING_KEYS_ITER = _descriptor.Descriptor(
  name='Iter',
  full_name='session.Thing.Keys.Iter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='session.Thing.Keys.Iter.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='id', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_THING_KEYS_ITER_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12980,
  serialized_end=13057,
)

_THING_KEYS = _descriptor.Descriptor(
  name='Keys',
  full_name='session.Thing.Keys',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_THING_KEYS_REQ, _THING_KEYS_ITER, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12907,
  serialized_end=13057,
)

_THING_ATTRIBUTES_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Thing.Attributes.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='attributeTypes', full_name='session.Thing.Attributes.Req.attributeTypes', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='attributeTypes', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12915,
  serialized_end=12978,
)

_THING_ATTRIBUTES_ITER_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Thing.Attributes.Iter.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='attribute', full_name='session.Thing.Attributes.Iter.Res.attribute', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='attribute', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12234,
  serialized_end=12287,
)

_THING_ATTRIBUTES_ITER = _descriptor.Descriptor(
  name='Iter',
  full_name='session.Thing.Attributes.Iter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='session.Thing.Attributes.Iter.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='id', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_THING_ATTRIBUTES_ITER_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12980,
  serialized_end=13057,
)

_THING_ATTRIBUTES = _descriptor.Descriptor(
  name='Attributes',
  full_name='session.Thing.Attributes',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_THING_ATTRIBUTES_REQ, _THING_ATTRIBUTES_ITER, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13060,
  serialized_end=13216,
)

_THING_RELATIONS_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Thing.Relations.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='roles', full_name='session.Thing.Relations.Req.roles', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='roles', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13232,
  serialized_end=13277,
)

_THING_RELATIONS_ITER_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Thing.Relations.Iter.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='relation', full_name='session.Thing.Relations.Iter.Res.relation', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='relation', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11890,
  serialized_end=11941,
)

_THING_RELATIONS_ITER = _descriptor.Descriptor(
  name='Iter',
  full_name='session.Thing.Relations.Iter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='session.Thing.Relations.Iter.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='id', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_THING_RELATIONS_ITER_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13279,
  serialized_end=13354,
)

_THING_RELATIONS = _descriptor.Descriptor(
  name='Relations',
  full_name='session.Thing.Relations',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_THING_RELATIONS_REQ, _THING_RELATIONS_ITER, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13219,
  serialized_end=13354,
)

_THING_ROLES_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Thing.Roles.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=51,
)

_THING_ROLES_ITER_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Thing.Roles.Iter.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='role', full_name='session.Thing.Roles.Iter.Res.role', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='role', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11296,
  serialized_end=11339,
)

_THING_ROLES_ITER = _descriptor.Descriptor(
  name='Iter',
  full_name='session.Thing.Roles.Iter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='session.Thing.Roles.Iter.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='id', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_THING_ROLES_ITER_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11272,
  serialized_end=11339,
)

_THING_ROLES = _descriptor.Descriptor(
  name='Roles',
  full_name='session.Thing.Roles',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_THING_ROLES_REQ, _THING_ROLES_ITER, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11943,
  serialized_end=12026,
)

_THING_RELHAS_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Thing.Relhas.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='attribute', full_name='session.Thing.Relhas.Req.attribute', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='attribute', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13451,
  serialized_end=13504,
)

_THING_RELHAS_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Thing.Relhas.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='relation', full_name='session.Thing.Relhas.Res.relation', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='relation', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11890,
  serialized_end=11941,
)

_THING_RELHAS = _descriptor.Descriptor(
  name='Relhas',
  full_name='session.Thing.Relhas',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_THING_RELHAS_REQ, _THING_RELHAS_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13441,
  serialized_end=13557,
)

_THING_UNHAS_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Thing.Unhas.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='attribute', full_name='session.Thing.Unhas.Req.attribute', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='attribute', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13451,
  serialized_end=13504,
)

_THING_UNHAS_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Thing.Unhas.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4041,
  serialized_end=4046,
)

_THING_UNHAS = _descriptor.Descriptor(
  name='Unhas',
  full_name='session.Thing.Unhas',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_THING_UNHAS_REQ, _THING_UNHAS_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13559,
  serialized_end=13628,
)

_THING = _descriptor.Descriptor(
  name='Thing',
  full_name='session.Thing',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_THING_ISINFERRED, _THING_TYPE, _THING_KEYS, _THING_ATTRIBUTES, _THING_RELATIONS, _THING_ROLES, _THING_RELHAS, _THING_UNHAS, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12781,
  serialized_end=13628,
)


_RELATION_ROLEPLAYERSMAP_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Relation.RolePlayersMap.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=51,
)

_RELATION_ROLEPLAYERSMAP_ITER_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Relation.RolePlayersMap.Iter.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='role', full_name='session.Relation.RolePlayersMap.Iter.Res.role', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='role', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='player', full_name='session.Relation.RolePlayersMap.Iter.Res.player', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='player', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13693,
  serialized_end=13778,
)

_RELATION_ROLEPLAYERSMAP_ITER = _descriptor.Descriptor(
  name='Iter',
  full_name='session.Relation.RolePlayersMap.Iter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='session.Relation.RolePlayersMap.Iter.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='id', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_RELATION_ROLEPLAYERSMAP_ITER_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13669,
  serialized_end=13778,
)

_RELATION_ROLEPLAYERSMAP = _descriptor.Descriptor(
  name='RolePlayersMap',
  full_name='session.Relation.RolePlayersMap',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_RELATION_ROLEPLAYERSMAP_REQ, _RELATION_ROLEPLAYERSMAP_ITER, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13644,
  serialized_end=13778,
)

_RELATION_ROLEPLAYERS_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Relation.RolePlayers.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='roles', full_name='session.Relation.RolePlayers.Req.roles', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='roles', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13232,
  serialized_end=13277,
)

_RELATION_ROLEPLAYERS_ITER_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Relation.RolePlayers.Iter.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='thing', full_name='session.Relation.RolePlayers.Iter.Res.thing', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='thing', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10997,
  serialized_end=11042,
)

_RELATION_ROLEPLAYERS_ITER = _descriptor.Descriptor(
  name='Iter',
  full_name='session.Relation.RolePlayers.Iter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='session.Relation.RolePlayers.Iter.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='id', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_RELATION_ROLEPLAYERS_ITER_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10973,
  serialized_end=11042,
)

_RELATION_ROLEPLAYERS = _descriptor.Descriptor(
  name='RolePlayers',
  full_name='session.Relation.RolePlayers',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_RELATION_ROLEPLAYERS_REQ, _RELATION_ROLEPLAYERS_ITER, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13781,
  serialized_end=13912,
)

_RELATION_ASSIGN_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Relation.Assign.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='role', full_name='session.Relation.Assign.Req.role', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='role', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='player', full_name='session.Relation.Assign.Req.player', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='player', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13924,
  serialized_end=14009,
)

_RELATION_ASSIGN_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Relation.Assign.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4041,
  serialized_end=4046,
)

_RELATION_ASSIGN = _descriptor.Descriptor(
  name='Assign',
  full_name='session.Relation.Assign',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_RELATION_ASSIGN_REQ, _RELATION_ASSIGN_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13914,
  serialized_end=14016,
)

_RELATION_UNASSIGN_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Relation.Unassign.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='role', full_name='session.Relation.Unassign.Req.role', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='role', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='player', full_name='session.Relation.Unassign.Req.player', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='player', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13924,
  serialized_end=14009,
)

_RELATION_UNASSIGN_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Relation.Unassign.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4041,
  serialized_end=4046,
)

_RELATION_UNASSIGN = _descriptor.Descriptor(
  name='Unassign',
  full_name='session.Relation.Unassign',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_RELATION_UNASSIGN_REQ, _RELATION_UNASSIGN_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=14018,
  serialized_end=14122,
)

_RELATION = _descriptor.Descriptor(
  name='Relation',
  full_name='session.Relation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_RELATION_ROLEPLAYERSMAP, _RELATION_ROLEPLAYERS, _RELATION_ASSIGN, _RELATION_UNASSIGN, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13631,
  serialized_end=14122,
)


_ATTRIBUTE_VALUE_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Attribute.Value.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=51,
)

_ATTRIBUTE_VALUE_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Attribute.Value.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='session.Attribute.Value.Res.value', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='value', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=14154,
  serialized_end=14203,
)

_ATTRIBUTE_VALUE = _descriptor.Descriptor(
  name='Value',
  full_name='session.Attribute.Value',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_ATTRIBUTE_VALUE_REQ, _ATTRIBUTE_VALUE_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=14138,
  serialized_end=14203,
)

_ATTRIBUTE_OWNERS_REQ = _descriptor.Descriptor(
  name='Req',
  full_name='session.Attribute.Owners.Req',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=51,
)

_ATTRIBUTE_OWNERS_ITER_RES = _descriptor.Descriptor(
  name='Res',
  full_name='session.Attribute.Owners.Iter.Res',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='thing', full_name='session.Attribute.Owners.Iter.Res.thing', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='thing', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10997,
  serialized_end=11042,
)

_ATTRIBUTE_OWNERS_ITER = _descriptor.Descriptor(
  name='Iter',
  full_name='session.Attribute.Owners.Iter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='session.Attribute.Owners.Iter.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='id', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_ATTRIBUTE_OWNERS_ITER_RES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10973,
  serialized_end=11042,
)

_ATTRIBUTE_OWNERS = _descriptor.Descriptor(
  name='Owners',
  full_name='session.Attribute.Owners',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_ATTRIBUTE_OWNERS_REQ, _ATTRIBUTE_OWNERS_ITER, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=14205,
  serialized_end=14291,
)

_ATTRIBUTE = _descriptor.Descriptor(
  name='Attribute',
  full_name='session.Attribute',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_ATTRIBUTE_VALUE, _ATTRIBUTE_OWNERS, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=14125,
  serialized_end=14291,
)


_VALUEOBJECT = _descriptor.Descriptor(
  name='ValueObject',
  full_name='session.ValueObject',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='string', full_name='session.ValueObject.string', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='string', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='boolean', full_name='session.ValueObject.boolean', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='boolean', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='integer', full_name='session.ValueObject.integer', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='integer', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='long', full_name='session.ValueObject.long', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='long', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='float', full_name='session.ValueObject.float', index=4,
      number=5, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='float', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='double', full_name='session.ValueObject.double', index=5,
      number=6, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='double', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='date', full_name='session.ValueObject.date', index=6,
      number=7, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='date', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='value', full_name='session.ValueObject.value',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=14294,
  serialized_end=14492,
)

_METHOD_REQ.fields_by_name['concept_delete_req'].message_type = _CONCEPT_DELETE_REQ
_METHOD_REQ.fields_by_name['schemaConcept_isImplicit_req'].message_type = _SCHEMACONCEPT_ISIMPLICIT_REQ
_METHOD_REQ.fields_by_name['schemaConcept_getLabel_req'].message_type = _SCHEMACONCEPT_GETLABEL_REQ
_METHOD_REQ.fields_by_name['schemaConcept_setLabel_req'].message_type = _SCHEMACONCEPT_SETLABEL_REQ
_METHOD_REQ.fields_by_name['schemaConcept_getSup_req'].message_type = _SCHEMACONCEPT_GETSUP_REQ
_METHOD_REQ.fields_by_name['schemaConcept_setSup_req'].message_type = _SCHEMACONCEPT_SETSUP_REQ
_METHOD_REQ.fields_by_name['schemaConcept_sups_req'].message_type = _SCHEMACONCEPT_SUPS_REQ
_METHOD_REQ.fields_by_name['schemaConcept_subs_req'].message_type = _SCHEMACONCEPT_SUBS_REQ
_METHOD_REQ.fields_by_name['rule_when_req'].message_type = _RULE_WHEN_REQ
_METHOD_REQ.fields_by_name['rule_then_req'].message_type = _RULE_THEN_REQ
_METHOD_REQ.fields_by_name['role_relations_req'].message_type = _ROLE_RELATIONS_REQ
_METHOD_REQ.fields_by_name['role_players_req'].message_type = _ROLE_PLAYERS_REQ
_METHOD_REQ.fields_by_name['type_isAbstract_req'].message_type = _TYPE_ISABSTRACT_REQ
_METHOD_REQ.fields_by_name['type_setAbstract_req'].message_type = _TYPE_SETABSTRACT_REQ
_METHOD_REQ.fields_by_name['type_instances_req'].message_type = _TYPE_INSTANCES_REQ
_METHOD_REQ.fields_by_name['type_keys_req'].message_type = _TYPE_KEYS_REQ
_METHOD_REQ.fields_by_name['type_attributes_req'].message_type = _TYPE_ATTRIBUTES_REQ
_METHOD_REQ.fields_by_name['type_playing_req'].message_type = _TYPE_PLAYING_REQ
_METHOD_REQ.fields_by_name['type_has_req'].message_type = _TYPE_HAS_REQ
_METHOD_REQ.fields_by_name['type_key_req'].message_type = _TYPE_KEY_REQ
_METHOD_REQ.fields_by_name['type_plays_req'].message_type = _TYPE_PLAYS_REQ
_METHOD_REQ.fields_by_name['type_unhas_req'].message_type = _TYPE_UNHAS_REQ
_METHOD_REQ.fields_by_name['type_unkey_req'].message_type = _TYPE_UNKEY_REQ
_METHOD_REQ.fields_by_name['type_unplay_req'].message_type = _TYPE_UNPLAY_REQ
_METHOD_REQ.fields_by_name['entityType_create_req'].message_type = _ENTITYTYPE_CREATE_REQ
_METHOD_REQ.fields_by_name['relationType_create_req'].message_type = _RELATIONTYPE_CREATE_REQ
_METHOD_REQ.fields_by_name['relationType_roles_req'].message_type = _RELATIONTYPE_ROLES_REQ
_METHOD_REQ.fields_by_name['relationType_relates_req'].message_type = _RELATIONTYPE_RELATES_REQ
_METHOD_REQ.fields_by_name['relationType_unrelate_req'].message_type = _RELATIONTYPE_UNRELATE_REQ
_METHOD_REQ.fields_by_name['attributeType_create_req'].message_type = _ATTRIBUTETYPE_CREATE_REQ
_METHOD_REQ.fields_by_name['attributeType_attribute_req'].message_type = _ATTRIBUTETYPE_ATTRIBUTE_REQ
_METHOD_REQ.fields_by_name['attributeType_dataType_req'].message_type = _ATTRIBUTETYPE_DATATYPE_REQ
_METHOD_REQ.fields_by_name['attributeType_getRegex_req'].message_type = _ATTRIBUTETYPE_GETREGEX_REQ
_METHOD_REQ.fields_by_name['attributeType_setRegex_req'].message_type = _ATTRIBUTETYPE_SETREGEX_REQ
_METHOD_REQ.fields_by_name['thing_type_req'].message_type = _THING_TYPE_REQ
_METHOD_REQ.fields_by_name['thing_isInferred_req'].message_type = _THING_ISINFERRED_REQ
_METHOD_REQ.fields_by_name['thing_keys_req'].message_type = _THING_KEYS_REQ
_METHOD_REQ.fields_by_name['thing_attributes_req'].message_type = _THING_ATTRIBUTES_REQ
_METHOD_REQ.fields_by_name['thing_relations_req'].message_type = _THING_RELATIONS_REQ
_METHOD_REQ.fields_by_name['thing_roles_req'].message_type = _THING_ROLES_REQ
_METHOD_REQ.fields_by_name['thing_relhas_req'].message_type = _THING_RELHAS_REQ
_METHOD_REQ.fields_by_name['thing_unhas_req'].message_type = _THING_UNHAS_REQ
_METHOD_REQ.fields_by_name['relation_rolePlayersMap_req'].message_type = _RELATION_ROLEPLAYERSMAP_REQ
_METHOD_REQ.fields_by_name['relation_rolePlayers_req'].message_type = _RELATION_ROLEPLAYERS_REQ
_METHOD_REQ.fields_by_name['relation_assign_req'].message_type = _RELATION_ASSIGN_REQ
_METHOD_REQ.fields_by_name['relation_unassign_req'].message_type = _RELATION_UNASSIGN_REQ
_METHOD_REQ.fields_by_name['attribute_value_req'].message_type = _ATTRIBUTE_VALUE_REQ
_METHOD_REQ.fields_by_name['attribute_owners_req'].message_type = _ATTRIBUTE_OWNERS_REQ
_METHOD_REQ.containing_type = _METHOD
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['concept_delete_req'])
_METHOD_REQ.fields_by_name['concept_delete_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['schemaConcept_isImplicit_req'])
_METHOD_REQ.fields_by_name['schemaConcept_isImplicit_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['schemaConcept_getLabel_req'])
_METHOD_REQ.fields_by_name['schemaConcept_getLabel_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['schemaConcept_setLabel_req'])
_METHOD_REQ.fields_by_name['schemaConcept_setLabel_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['schemaConcept_getSup_req'])
_METHOD_REQ.fields_by_name['schemaConcept_getSup_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['schemaConcept_setSup_req'])
_METHOD_REQ.fields_by_name['schemaConcept_setSup_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['schemaConcept_sups_req'])
_METHOD_REQ.fields_by_name['schemaConcept_sups_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['schemaConcept_subs_req'])
_METHOD_REQ.fields_by_name['schemaConcept_subs_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['rule_when_req'])
_METHOD_REQ.fields_by_name['rule_when_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['rule_then_req'])
_METHOD_REQ.fields_by_name['rule_then_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['role_relations_req'])
_METHOD_REQ.fields_by_name['role_relations_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['role_players_req'])
_METHOD_REQ.fields_by_name['role_players_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['type_isAbstract_req'])
_METHOD_REQ.fields_by_name['type_isAbstract_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['type_setAbstract_req'])
_METHOD_REQ.fields_by_name['type_setAbstract_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['type_instances_req'])
_METHOD_REQ.fields_by_name['type_instances_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['type_keys_req'])
_METHOD_REQ.fields_by_name['type_keys_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['type_attributes_req'])
_METHOD_REQ.fields_by_name['type_attributes_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['type_playing_req'])
_METHOD_REQ.fields_by_name['type_playing_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['type_has_req'])
_METHOD_REQ.fields_by_name['type_has_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['type_key_req'])
_METHOD_REQ.fields_by_name['type_key_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['type_plays_req'])
_METHOD_REQ.fields_by_name['type_plays_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['type_unhas_req'])
_METHOD_REQ.fields_by_name['type_unhas_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['type_unkey_req'])
_METHOD_REQ.fields_by_name['type_unkey_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['type_unplay_req'])
_METHOD_REQ.fields_by_name['type_unplay_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['entityType_create_req'])
_METHOD_REQ.fields_by_name['entityType_create_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['relationType_create_req'])
_METHOD_REQ.fields_by_name['relationType_create_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['relationType_roles_req'])
_METHOD_REQ.fields_by_name['relationType_roles_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['relationType_relates_req'])
_METHOD_REQ.fields_by_name['relationType_relates_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['relationType_unrelate_req'])
_METHOD_REQ.fields_by_name['relationType_unrelate_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['attributeType_create_req'])
_METHOD_REQ.fields_by_name['attributeType_create_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['attributeType_attribute_req'])
_METHOD_REQ.fields_by_name['attributeType_attribute_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['attributeType_dataType_req'])
_METHOD_REQ.fields_by_name['attributeType_dataType_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['attributeType_getRegex_req'])
_METHOD_REQ.fields_by_name['attributeType_getRegex_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['attributeType_setRegex_req'])
_METHOD_REQ.fields_by_name['attributeType_setRegex_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['thing_type_req'])
_METHOD_REQ.fields_by_name['thing_type_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['thing_isInferred_req'])
_METHOD_REQ.fields_by_name['thing_isInferred_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['thing_keys_req'])
_METHOD_REQ.fields_by_name['thing_keys_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['thing_attributes_req'])
_METHOD_REQ.fields_by_name['thing_attributes_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['thing_relations_req'])
_METHOD_REQ.fields_by_name['thing_relations_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['thing_roles_req'])
_METHOD_REQ.fields_by_name['thing_roles_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['thing_relhas_req'])
_METHOD_REQ.fields_by_name['thing_relhas_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['thing_unhas_req'])
_METHOD_REQ.fields_by_name['thing_unhas_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['relation_rolePlayersMap_req'])
_METHOD_REQ.fields_by_name['relation_rolePlayersMap_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['relation_rolePlayers_req'])
_METHOD_REQ.fields_by_name['relation_rolePlayers_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['relation_assign_req'])
_METHOD_REQ.fields_by_name['relation_assign_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['relation_unassign_req'])
_METHOD_REQ.fields_by_name['relation_unassign_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['attribute_value_req'])
_METHOD_REQ.fields_by_name['attribute_value_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_REQ.oneofs_by_name['req'].fields.append(
  _METHOD_REQ.fields_by_name['attribute_owners_req'])
_METHOD_REQ.fields_by_name['attribute_owners_req'].containing_oneof = _METHOD_REQ.oneofs_by_name['req']
_METHOD_RES.fields_by_name['concept_delete_res'].message_type = _CONCEPT_DELETE_RES
_METHOD_RES.fields_by_name['schemaConcept_isImplicit_res'].message_type = _SCHEMACONCEPT_ISIMPLICIT_RES
_METHOD_RES.fields_by_name['schemaConcept_getLabel_res'].message_type = _SCHEMACONCEPT_GETLABEL_RES
_METHOD_RES.fields_by_name['schemaConcept_setLabel_res'].message_type = _SCHEMACONCEPT_SETLABEL_RES
_METHOD_RES.fields_by_name['schemaConcept_getSup_res'].message_type = _SCHEMACONCEPT_GETSUP_RES
_METHOD_RES.fields_by_name['schemaConcept_setSup_res'].message_type = _SCHEMACONCEPT_SETSUP_RES
_METHOD_RES.fields_by_name['schemaConcept_sups_iter'].message_type = _SCHEMACONCEPT_SUPS_ITER
_METHOD_RES.fields_by_name['schemaConcept_subs_iter'].message_type = _SCHEMACONCEPT_SUBS_ITER
_METHOD_RES.fields_by_name['rule_when_res'].message_type = _RULE_WHEN_RES
_METHOD_RES.fields_by_name['rule_then_res'].message_type = _RULE_THEN_RES
_METHOD_RES.fields_by_name['role_relations_iter'].message_type = _ROLE_RELATIONS_ITER
_METHOD_RES.fields_by_name['role_players_iter'].message_type = _ROLE_PLAYERS_ITER
_METHOD_RES.fields_by_name['type_isAbstract_res'].message_type = _TYPE_ISABSTRACT_RES
_METHOD_RES.fields_by_name['type_setAbstract_res'].message_type = _TYPE_SETABSTRACT_RES
_METHOD_RES.fields_by_name['type_instances_iter'].message_type = _TYPE_INSTANCES_ITER
_METHOD_RES.fields_by_name['type_keys_iter'].message_type = _TYPE_KEYS_ITER
_METHOD_RES.fields_by_name['type_attributes_iter'].message_type = _TYPE_ATTRIBUTES_ITER
_METHOD_RES.fields_by_name['type_playing_iter'].message_type = _TYPE_PLAYING_ITER
_METHOD_RES.fields_by_name['type_has_res'].message_type = _TYPE_HAS_RES
_METHOD_RES.fields_by_name['type_key_res'].message_type = _TYPE_KEY_RES
_METHOD_RES.fields_by_name['type_plays_res'].message_type = _TYPE_PLAYS_RES
_METHOD_RES.fields_by_name['type_unhas_res'].message_type = _TYPE_UNHAS_RES
_METHOD_RES.fields_by_name['type_unkey_res'].message_type = _TYPE_UNKEY_RES
_METHOD_RES.fields_by_name['type_unplay_res'].message_type = _TYPE_UNPLAY_RES
_METHOD_RES.fields_by_name['entityType_create_res'].message_type = _ENTITYTYPE_CREATE_RES
_METHOD_RES.fields_by_name['relationType_create_res'].message_type = _RELATIONTYPE_CREATE_RES
_METHOD_RES.fields_by_name['relationType_roles_iter'].message_type = _RELATIONTYPE_ROLES_ITER
_METHOD_RES.fields_by_name['relationType_relates_res'].message_type = _RELATIONTYPE_RELATES_RES
_METHOD_RES.fields_by_name['relationType_unrelate_res'].message_type = _RELATIONTYPE_UNRELATE_RES
_METHOD_RES.fields_by_name['attributeType_create_res'].message_type = _ATTRIBUTETYPE_CREATE_RES
_METHOD_RES.fields_by_name['attributeType_attribute_res'].message_type = _ATTRIBUTETYPE_ATTRIBUTE_RES
_METHOD_RES.fields_by_name['attributeType_dataType_res'].message_type = _ATTRIBUTETYPE_DATATYPE_RES
_METHOD_RES.fields_by_name['attributeType_getRegex_res'].message_type = _ATTRIBUTETYPE_GETREGEX_RES
_METHOD_RES.fields_by_name['attributeType_setRegex_res'].message_type = _ATTRIBUTETYPE_SETREGEX_RES
_METHOD_RES.fields_by_name['thing_type_res'].message_type = _THING_TYPE_RES
_METHOD_RES.fields_by_name['thing_isInferred_res'].message_type = _THING_ISINFERRED_RES
_METHOD_RES.fields_by_name['thing_keys_iter'].message_type = _THING_KEYS_ITER
_METHOD_RES.fields_by_name['thing_attributes_iter'].message_type = _THING_ATTRIBUTES_ITER
_METHOD_RES.fields_by_name['thing_relations_iter'].message_type = _THING_RELATIONS_ITER
_METHOD_RES.fields_by_name['thing_roles_iter'].message_type = _THING_ROLES_ITER
_METHOD_RES.fields_by_name['thing_relhas_res'].message_type = _THING_RELHAS_RES
_METHOD_RES.fields_by_name['thing_unhas_res'].message_type = _THING_UNHAS_RES
_METHOD_RES.fields_by_name['relation_rolePlayersMap_iter'].message_type = _RELATION_ROLEPLAYERSMAP_ITER
_METHOD_RES.fields_by_name['relation_rolePlayers_iter'].message_type = _RELATION_ROLEPLAYERS_ITER
_METHOD_RES.fields_by_name['relation_assign_res'].message_type = _RELATION_ASSIGN_RES
_METHOD_RES.fields_by_name['relation_unassign_res'].message_type = _RELATION_UNASSIGN_RES
_METHOD_RES.fields_by_name['attribute_value_res'].message_type = _ATTRIBUTE_VALUE_RES
_METHOD_RES.fields_by_name['attribute_owners_iter'].message_type = _ATTRIBUTE_OWNERS_ITER
_METHOD_RES.containing_type = _METHOD
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['concept_delete_res'])
_METHOD_RES.fields_by_name['concept_delete_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['schemaConcept_isImplicit_res'])
_METHOD_RES.fields_by_name['schemaConcept_isImplicit_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['schemaConcept_getLabel_res'])
_METHOD_RES.fields_by_name['schemaConcept_getLabel_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['schemaConcept_setLabel_res'])
_METHOD_RES.fields_by_name['schemaConcept_setLabel_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['schemaConcept_getSup_res'])
_METHOD_RES.fields_by_name['schemaConcept_getSup_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['schemaConcept_setSup_res'])
_METHOD_RES.fields_by_name['schemaConcept_setSup_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['schemaConcept_sups_iter'])
_METHOD_RES.fields_by_name['schemaConcept_sups_iter'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['schemaConcept_subs_iter'])
_METHOD_RES.fields_by_name['schemaConcept_subs_iter'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['rule_when_res'])
_METHOD_RES.fields_by_name['rule_when_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['rule_then_res'])
_METHOD_RES.fields_by_name['rule_then_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['role_relations_iter'])
_METHOD_RES.fields_by_name['role_relations_iter'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['role_players_iter'])
_METHOD_RES.fields_by_name['role_players_iter'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['type_isAbstract_res'])
_METHOD_RES.fields_by_name['type_isAbstract_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['type_setAbstract_res'])
_METHOD_RES.fields_by_name['type_setAbstract_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['type_instances_iter'])
_METHOD_RES.fields_by_name['type_instances_iter'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['type_keys_iter'])
_METHOD_RES.fields_by_name['type_keys_iter'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['type_attributes_iter'])
_METHOD_RES.fields_by_name['type_attributes_iter'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['type_playing_iter'])
_METHOD_RES.fields_by_name['type_playing_iter'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['type_has_res'])
_METHOD_RES.fields_by_name['type_has_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['type_key_res'])
_METHOD_RES.fields_by_name['type_key_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['type_plays_res'])
_METHOD_RES.fields_by_name['type_plays_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['type_unhas_res'])
_METHOD_RES.fields_by_name['type_unhas_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['type_unkey_res'])
_METHOD_RES.fields_by_name['type_unkey_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['type_unplay_res'])
_METHOD_RES.fields_by_name['type_unplay_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['entityType_create_res'])
_METHOD_RES.fields_by_name['entityType_create_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['relationType_create_res'])
_METHOD_RES.fields_by_name['relationType_create_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['relationType_roles_iter'])
_METHOD_RES.fields_by_name['relationType_roles_iter'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['relationType_relates_res'])
_METHOD_RES.fields_by_name['relationType_relates_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['relationType_unrelate_res'])
_METHOD_RES.fields_by_name['relationType_unrelate_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['attributeType_create_res'])
_METHOD_RES.fields_by_name['attributeType_create_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['attributeType_attribute_res'])
_METHOD_RES.fields_by_name['attributeType_attribute_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['attributeType_dataType_res'])
_METHOD_RES.fields_by_name['attributeType_dataType_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['attributeType_getRegex_res'])
_METHOD_RES.fields_by_name['attributeType_getRegex_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['attributeType_setRegex_res'])
_METHOD_RES.fields_by_name['attributeType_setRegex_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['thing_type_res'])
_METHOD_RES.fields_by_name['thing_type_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['thing_isInferred_res'])
_METHOD_RES.fields_by_name['thing_isInferred_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['thing_keys_iter'])
_METHOD_RES.fields_by_name['thing_keys_iter'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['thing_attributes_iter'])
_METHOD_RES.fields_by_name['thing_attributes_iter'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['thing_relations_iter'])
_METHOD_RES.fields_by_name['thing_relations_iter'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['thing_roles_iter'])
_METHOD_RES.fields_by_name['thing_roles_iter'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['thing_relhas_res'])
_METHOD_RES.fields_by_name['thing_relhas_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['thing_unhas_res'])
_METHOD_RES.fields_by_name['thing_unhas_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['relation_rolePlayersMap_iter'])
_METHOD_RES.fields_by_name['relation_rolePlayersMap_iter'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['relation_rolePlayers_iter'])
_METHOD_RES.fields_by_name['relation_rolePlayers_iter'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['relation_assign_res'])
_METHOD_RES.fields_by_name['relation_assign_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['relation_unassign_res'])
_METHOD_RES.fields_by_name['relation_unassign_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['attribute_value_res'])
_METHOD_RES.fields_by_name['attribute_value_res'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_RES.oneofs_by_name['res'].fields.append(
  _METHOD_RES.fields_by_name['attribute_owners_iter'])
_METHOD_RES.fields_by_name['attribute_owners_iter'].containing_oneof = _METHOD_RES.oneofs_by_name['res']
_METHOD_ITER_RES.fields_by_name['schemaConcept_sups_iter_res'].message_type = _SCHEMACONCEPT_SUPS_ITER_RES
_METHOD_ITER_RES.fields_by_name['schemaConcept_subs_iter_res'].message_type = _SCHEMACONCEPT_SUBS_ITER_RES
_METHOD_ITER_RES.fields_by_name['role_relations_iter_res'].message_type = _ROLE_RELATIONS_ITER_RES
_METHOD_ITER_RES.fields_by_name['role_players_iter_res'].message_type = _ROLE_PLAYERS_ITER_RES
_METHOD_ITER_RES.fields_by_name['type_instances_iter_res'].message_type = _TYPE_INSTANCES_ITER_RES
_METHOD_ITER_RES.fields_by_name['type_keys_iter_res'].message_type = _TYPE_KEYS_ITER_RES
_METHOD_ITER_RES.fields_by_name['type_attributes_iter_res'].message_type = _TYPE_ATTRIBUTES_ITER_RES
_METHOD_ITER_RES.fields_by_name['type_playing_iter_res'].message_type = _TYPE_PLAYING_ITER_RES
_METHOD_ITER_RES.fields_by_name['relationType_roles_iter_res'].message_type = _RELATIONTYPE_ROLES_ITER_RES
_METHOD_ITER_RES.fields_by_name['thing_keys_iter_res'].message_type = _THING_KEYS_ITER_RES
_METHOD_ITER_RES.fields_by_name['thing_attributes_iter_res'].message_type = _THING_ATTRIBUTES_ITER_RES
_METHOD_ITER_RES.fields_by_name['thing_relations_iter_res'].message_type = _THING_RELATIONS_ITER_RES
_METHOD_ITER_RES.fields_by_name['thing_roles_iter_res'].message_type = _THING_ROLES_ITER_RES
_METHOD_ITER_RES.fields_by_name['relation_rolePlayersMap_iter_res'].message_type = _RELATION_ROLEPLAYERSMAP_ITER_RES
_METHOD_ITER_RES.fields_by_name['relation_rolePlayers_iter_res'].message_type = _RELATION_ROLEPLAYERS_ITER_RES
_METHOD_ITER_RES.fields_by_name['attribute_owners_iter_res'].message_type = _ATTRIBUTE_OWNERS_ITER_RES
_METHOD_ITER_RES.containing_type = _METHOD_ITER
_METHOD_ITER.containing_type = _METHOD
_CONCEPT_DELETE_REQ.containing_type = _CONCEPT_DELETE
_CONCEPT_DELETE_RES.containing_type = _CONCEPT_DELETE
_CONCEPT_DELETE.containing_type = _CONCEPT
_CONCEPT.fields_by_name['baseType'].enum_type = _CONCEPT_BASE_TYPE
_CONCEPT_BASE_TYPE.containing_type = _CONCEPT
_SCHEMACONCEPT_GETLABEL_REQ.containing_type = _SCHEMACONCEPT_GETLABEL
_SCHEMACONCEPT_GETLABEL_RES.containing_type = _SCHEMACONCEPT_GETLABEL
_SCHEMACONCEPT_GETLABEL.containing_type = _SCHEMACONCEPT
_SCHEMACONCEPT_SETLABEL_REQ.containing_type = _SCHEMACONCEPT_SETLABEL
_SCHEMACONCEPT_SETLABEL_RES.containing_type = _SCHEMACONCEPT_SETLABEL
_SCHEMACONCEPT_SETLABEL.containing_type = _SCHEMACONCEPT
_SCHEMACONCEPT_ISIMPLICIT_REQ.containing_type = _SCHEMACONCEPT_ISIMPLICIT
_SCHEMACONCEPT_ISIMPLICIT_RES.containing_type = _SCHEMACONCEPT_ISIMPLICIT
_SCHEMACONCEPT_ISIMPLICIT.containing_type = _SCHEMACONCEPT
_SCHEMACONCEPT_GETSUP_REQ.containing_type = _SCHEMACONCEPT_GETSUP
_SCHEMACONCEPT_GETSUP_RES.fields_by_name['schemaConcept'].message_type = _CONCEPT
_SCHEMACONCEPT_GETSUP_RES.fields_by_name['null'].message_type = _NULL
_SCHEMACONCEPT_GETSUP_RES.containing_type = _SCHEMACONCEPT_GETSUP
_SCHEMACONCEPT_GETSUP_RES.oneofs_by_name['res'].fields.append(
  _SCHEMACONCEPT_GETSUP_RES.fields_by_name['schemaConcept'])
_SCHEMACONCEPT_GETSUP_RES.fields_by_name['schemaConcept'].containing_oneof = _SCHEMACONCEPT_GETSUP_RES.oneofs_by_name['res']
_SCHEMACONCEPT_GETSUP_RES.oneofs_by_name['res'].fields.append(
  _SCHEMACONCEPT_GETSUP_RES.fields_by_name['null'])
_SCHEMACONCEPT_GETSUP_RES.fields_by_name['null'].containing_oneof = _SCHEMACONCEPT_GETSUP_RES.oneofs_by_name['res']
_SCHEMACONCEPT_GETSUP.containing_type = _SCHEMACONCEPT
_SCHEMACONCEPT_SETSUP_REQ.fields_by_name['schemaConcept'].message_type = _CONCEPT
_SCHEMACONCEPT_SETSUP_REQ.containing_type = _SCHEMACONCEPT_SETSUP
_SCHEMACONCEPT_SETSUP_RES.containing_type = _SCHEMACONCEPT_SETSUP
_SCHEMACONCEPT_SETSUP.containing_type = _SCHEMACONCEPT
_SCHEMACONCEPT_SUPS_REQ.containing_type = _SCHEMACONCEPT_SUPS
_SCHEMACONCEPT_SUPS_ITER_RES.fields_by_name['schemaConcept'].message_type = _CONCEPT
_SCHEMACONCEPT_SUPS_ITER_RES.containing_type = _SCHEMACONCEPT_SUPS_ITER
_SCHEMACONCEPT_SUPS_ITER.containing_type = _SCHEMACONCEPT_SUPS
_SCHEMACONCEPT_SUPS.containing_type = _SCHEMACONCEPT
_SCHEMACONCEPT_SUBS_REQ.containing_type = _SCHEMACONCEPT_SUBS
_SCHEMACONCEPT_SUBS_ITER_RES.fields_by_name['schemaConcept'].message_type = _CONCEPT
_SCHEMACONCEPT_SUBS_ITER_RES.containing_type = _SCHEMACONCEPT_SUBS_ITER
_SCHEMACONCEPT_SUBS_ITER.containing_type = _SCHEMACONCEPT_SUBS
_SCHEMACONCEPT_SUBS.containing_type = _SCHEMACONCEPT
_RULE_WHEN_REQ.containing_type = _RULE_WHEN
_RULE_WHEN_RES.fields_by_name['null'].message_type = _NULL
_RULE_WHEN_RES.containing_type = _RULE_WHEN
_RULE_WHEN_RES.oneofs_by_name['res'].fields.append(
  _RULE_WHEN_RES.fields_by_name['pattern'])
_RULE_WHEN_RES.fields_by_name['pattern'].containing_oneof = _RULE_WHEN_RES.oneofs_by_name['res']
_RULE_WHEN_RES.oneofs_by_name['res'].fields.append(
  _RULE_WHEN_RES.fields_by_name['null'])
_RULE_WHEN_RES.fields_by_name['null'].containing_oneof = _RULE_WHEN_RES.oneofs_by_name['res']
_RULE_WHEN.containing_type = _RULE
_RULE_THEN_REQ.containing_type = _RULE_THEN
_RULE_THEN_RES.fields_by_name['null'].message_type = _NULL
_RULE_THEN_RES.containing_type = _RULE_THEN
_RULE_THEN_RES.oneofs_by_name['res'].fields.append(
  _RULE_THEN_RES.fields_by_name['pattern'])
_RULE_THEN_RES.fields_by_name['pattern'].containing_oneof = _RULE_THEN_RES.oneofs_by_name['res']
_RULE_THEN_RES.oneofs_by_name['res'].fields.append(
  _RULE_THEN_RES.fields_by_name['null'])
_RULE_THEN_RES.fields_by_name['null'].containing_oneof = _RULE_THEN_RES.oneofs_by_name['res']
_RULE_THEN.containing_type = _RULE
_ROLE_RELATIONS_REQ.containing_type = _ROLE_RELATIONS
_ROLE_RELATIONS_ITER_RES.fields_by_name['relationType'].message_type = _CONCEPT
_ROLE_RELATIONS_ITER_RES.containing_type = _ROLE_RELATIONS_ITER
_ROLE_RELATIONS_ITER.containing_type = _ROLE_RELATIONS
_ROLE_RELATIONS.containing_type = _ROLE
_ROLE_PLAYERS_REQ.containing_type = _ROLE_PLAYERS
_ROLE_PLAYERS_ITER_RES.fields_by_name['type'].message_type = _CONCEPT
_ROLE_PLAYERS_ITER_RES.containing_type = _ROLE_PLAYERS_ITER
_ROLE_PLAYERS_ITER.containing_type = _ROLE_PLAYERS
_ROLE_PLAYERS.containing_type = _ROLE
_TYPE_ISABSTRACT_REQ.containing_type = _TYPE_ISABSTRACT
_TYPE_ISABSTRACT_RES.containing_type = _TYPE_ISABSTRACT
_TYPE_ISABSTRACT.containing_type = _TYPE
_TYPE_SETABSTRACT_REQ.containing_type = _TYPE_SETABSTRACT
_TYPE_SETABSTRACT_RES.containing_type = _TYPE_SETABSTRACT
_TYPE_SETABSTRACT.containing_type = _TYPE
_TYPE_INSTANCES_REQ.containing_type = _TYPE_INSTANCES
_TYPE_INSTANCES_ITER_RES.fields_by_name['thing'].message_type = _CONCEPT
_TYPE_INSTANCES_ITER_RES.containing_type = _TYPE_INSTANCES_ITER
_TYPE_INSTANCES_ITER.containing_type = _TYPE_INSTANCES
_TYPE_INSTANCES.containing_type = _TYPE
_TYPE_ATTRIBUTES_REQ.containing_type = _TYPE_ATTRIBUTES
_TYPE_ATTRIBUTES_ITER_RES.fields_by_name['attributeType'].message_type = _CONCEPT
_TYPE_ATTRIBUTES_ITER_RES.containing_type = _TYPE_ATTRIBUTES_ITER
_TYPE_ATTRIBUTES_ITER.containing_type = _TYPE_ATTRIBUTES
_TYPE_ATTRIBUTES.containing_type = _TYPE
_TYPE_KEYS_REQ.containing_type = _TYPE_KEYS
_TYPE_KEYS_ITER_RES.fields_by_name['attributeType'].message_type = _CONCEPT
_TYPE_KEYS_ITER_RES.containing_type = _TYPE_KEYS_ITER
_TYPE_KEYS_ITER.containing_type = _TYPE_KEYS
_TYPE_KEYS.containing_type = _TYPE
_TYPE_PLAYING_REQ.containing_type = _TYPE_PLAYING
_TYPE_PLAYING_ITER_RES.fields_by_name['role'].message_type = _CONCEPT
_TYPE_PLAYING_ITER_RES.containing_type = _TYPE_PLAYING_ITER
_TYPE_PLAYING_ITER.containing_type = _TYPE_PLAYING
_TYPE_PLAYING.containing_type = _TYPE
_TYPE_KEY_REQ.fields_by_name['attributeType'].message_type = _CONCEPT
_TYPE_KEY_REQ.containing_type = _TYPE_KEY
_TYPE_KEY_RES.containing_type = _TYPE_KEY
_TYPE_KEY.containing_type = _TYPE
_TYPE_HAS_REQ.fields_by_name['attributeType'].message_type = _CONCEPT
_TYPE_HAS_REQ.containing_type = _TYPE_HAS
_TYPE_HAS_RES.containing_type = _TYPE_HAS
_TYPE_HAS.containing_type = _TYPE
_TYPE_PLAYS_REQ.fields_by_name['role'].message_type = _CONCEPT
_TYPE_PLAYS_REQ.containing_type = _TYPE_PLAYS
_TYPE_PLAYS_RES.containing_type = _TYPE_PLAYS
_TYPE_PLAYS.containing_type = _TYPE
_TYPE_UNKEY_REQ.fields_by_name['attributeType'].message_type = _CONCEPT
_TYPE_UNKEY_REQ.containing_type = _TYPE_UNKEY
_TYPE_UNKEY_RES.containing_type = _TYPE_UNKEY
_TYPE_UNKEY.containing_type = _TYPE
_TYPE_UNHAS_REQ.fields_by_name['attributeType'].message_type = _CONCEPT
_TYPE_UNHAS_REQ.containing_type = _TYPE_UNHAS
_TYPE_UNHAS_RES.containing_type = _TYPE_UNHAS
_TYPE_UNHAS.containing_type = _TYPE
_TYPE_UNPLAY_REQ.fields_by_name['role'].message_type = _CONCEPT
_TYPE_UNPLAY_REQ.containing_type = _TYPE_UNPLAY
_TYPE_UNPLAY_RES.containing_type = _TYPE_UNPLAY
_TYPE_UNPLAY.containing_type = _TYPE
_ENTITYTYPE_CREATE_REQ.containing_type = _ENTITYTYPE_CREATE
_ENTITYTYPE_CREATE_RES.fields_by_name['entity'].message_type = _CONCEPT
_ENTITYTYPE_CREATE_RES.containing_type = _ENTITYTYPE_CREATE
_ENTITYTYPE_CREATE.containing_type = _ENTITYTYPE
_RELATIONTYPE_CREATE_REQ.containing_type = _RELATIONTYPE_CREATE
_RELATIONTYPE_CREATE_RES.fields_by_name['relation'].message_type = _CONCEPT
_RELATIONTYPE_CREATE_RES.containing_type = _RELATIONTYPE_CREATE
_RELATIONTYPE_CREATE.containing_type = _RELATIONTYPE
_RELATIONTYPE_ROLES_REQ.containing_type = _RELATIONTYPE_ROLES
_RELATIONTYPE_ROLES_ITER_RES.fields_by_name['role'].message_type = _CONCEPT
_RELATIONTYPE_ROLES_ITER_RES.containing_type = _RELATIONTYPE_ROLES_ITER
_RELATIONTYPE_ROLES_ITER.containing_type = _RELATIONTYPE_ROLES
_RELATIONTYPE_ROLES.containing_type = _RELATIONTYPE
_RELATIONTYPE_RELATES_REQ.fields_by_name['role'].message_type = _CONCEPT
_RELATIONTYPE_RELATES_REQ.containing_type = _RELATIONTYPE_RELATES
_RELATIONTYPE_RELATES_RES.containing_type = _RELATIONTYPE_RELATES
_RELATIONTYPE_RELATES.containing_type = _RELATIONTYPE
_RELATIONTYPE_UNRELATE_REQ.fields_by_name['role'].message_type = _CONCEPT
_RELATIONTYPE_UNRELATE_REQ.containing_type = _RELATIONTYPE_UNRELATE
_RELATIONTYPE_UNRELATE_RES.containing_type = _RELATIONTYPE_UNRELATE
_RELATIONTYPE_UNRELATE.containing_type = _RELATIONTYPE
_ATTRIBUTETYPE_CREATE_REQ.fields_by_name['value'].message_type = _VALUEOBJECT
_ATTRIBUTETYPE_CREATE_REQ.containing_type = _ATTRIBUTETYPE_CREATE
_ATTRIBUTETYPE_CREATE_RES.fields_by_name['attribute'].message_type = _CONCEPT
_ATTRIBUTETYPE_CREATE_RES.containing_type = _ATTRIBUTETYPE_CREATE
_ATTRIBUTETYPE_CREATE.containing_type = _ATTRIBUTETYPE
_ATTRIBUTETYPE_ATTRIBUTE_REQ.fields_by_name['value'].message_type = _VALUEOBJECT
_ATTRIBUTETYPE_ATTRIBUTE_REQ.containing_type = _ATTRIBUTETYPE_ATTRIBUTE
_ATTRIBUTETYPE_ATTRIBUTE_RES.fields_by_name['attribute'].message_type = _CONCEPT
_ATTRIBUTETYPE_ATTRIBUTE_RES.fields_by_name['null'].message_type = _NULL
_ATTRIBUTETYPE_ATTRIBUTE_RES.containing_type = _ATTRIBUTETYPE_ATTRIBUTE
_ATTRIBUTETYPE_ATTRIBUTE_RES.oneofs_by_name['res'].fields.append(
  _ATTRIBUTETYPE_ATTRIBUTE_RES.fields_by_name['attribute'])
_ATTRIBUTETYPE_ATTRIBUTE_RES.fields_by_name['attribute'].containing_oneof = _ATTRIBUTETYPE_ATTRIBUTE_RES.oneofs_by_name['res']
_ATTRIBUTETYPE_ATTRIBUTE_RES.oneofs_by_name['res'].fields.append(
  _ATTRIBUTETYPE_ATTRIBUTE_RES.fields_by_name['null'])
_ATTRIBUTETYPE_ATTRIBUTE_RES.fields_by_name['null'].containing_oneof = _ATTRIBUTETYPE_ATTRIBUTE_RES.oneofs_by_name['res']
_ATTRIBUTETYPE_ATTRIBUTE.containing_type = _ATTRIBUTETYPE
_ATTRIBUTETYPE_DATATYPE_REQ.containing_type = _ATTRIBUTETYPE_DATATYPE
_ATTRIBUTETYPE_DATATYPE_RES.fields_by_name['dataType'].enum_type = _ATTRIBUTETYPE_DATA_TYPE
_ATTRIBUTETYPE_DATATYPE_RES.fields_by_name['null'].message_type = _NULL
_ATTRIBUTETYPE_DATATYPE_RES.containing_type = _ATTRIBUTETYPE_DATATYPE
_ATTRIBUTETYPE_DATATYPE_RES.oneofs_by_name['res'].fields.append(
  _ATTRIBUTETYPE_DATATYPE_RES.fields_by_name['dataType'])
_ATTRIBUTETYPE_DATATYPE_RES.fields_by_name['dataType'].containing_oneof = _ATTRIBUTETYPE_DATATYPE_RES.oneofs_by_name['res']
_ATTRIBUTETYPE_DATATYPE_RES.oneofs_by_name['res'].fields.append(
  _ATTRIBUTETYPE_DATATYPE_RES.fields_by_name['null'])
_ATTRIBUTETYPE_DATATYPE_RES.fields_by_name['null'].containing_oneof = _ATTRIBUTETYPE_DATATYPE_RES.oneofs_by_name['res']
_ATTRIBUTETYPE_DATATYPE.containing_type = _ATTRIBUTETYPE
_ATTRIBUTETYPE_GETREGEX_REQ.containing_type = _ATTRIBUTETYPE_GETREGEX
_ATTRIBUTETYPE_GETREGEX_RES.containing_type = _ATTRIBUTETYPE_GETREGEX
_ATTRIBUTETYPE_GETREGEX.containing_type = _ATTRIBUTETYPE
_ATTRIBUTETYPE_SETREGEX_REQ.containing_type = _ATTRIBUTETYPE_SETREGEX
_ATTRIBUTETYPE_SETREGEX_RES.containing_type = _ATTRIBUTETYPE_SETREGEX
_ATTRIBUTETYPE_SETREGEX.containing_type = _ATTRIBUTETYPE
_ATTRIBUTETYPE_DATA_TYPE.containing_type = _ATTRIBUTETYPE
_THING_ISINFERRED_REQ.containing_type = _THING_ISINFERRED
_THING_ISINFERRED_RES.containing_type = _THING_ISINFERRED
_THING_ISINFERRED.containing_type = _THING
_THING_TYPE_REQ.containing_type = _THING_TYPE
_THING_TYPE_RES.fields_by_name['type'].message_type = _CONCEPT
_THING_TYPE_RES.containing_type = _THING_TYPE
_THING_TYPE.containing_type = _THING
_THING_KEYS_REQ.fields_by_name['attributeTypes'].message_type = _CONCEPT
_THING_KEYS_REQ.containing_type = _THING_KEYS
_THING_KEYS_ITER_RES.fields_by_name['attribute'].message_type = _CONCEPT
_THING_KEYS_ITER_RES.containing_type = _THING_KEYS_ITER
_THING_KEYS_ITER.containing_type = _THING_KEYS
_THING_KEYS.containing_type = _THING
_THING_ATTRIBUTES_REQ.fields_by_name['attributeTypes'].message_type = _CONCEPT
_THING_ATTRIBUTES_REQ.containing_type = _THING_ATTRIBUTES
_THING_ATTRIBUTES_ITER_RES.fields_by_name['attribute'].message_type = _CONCEPT
_THING_ATTRIBUTES_ITER_RES.containing_type = _THING_ATTRIBUTES_ITER
_THING_ATTRIBUTES_ITER.containing_type = _THING_ATTRIBUTES
_THING_ATTRIBUTES.containing_type = _THING
_THING_RELATIONS_REQ.fields_by_name['roles'].message_type = _CONCEPT
_THING_RELATIONS_REQ.containing_type = _THING_RELATIONS
_THING_RELATIONS_ITER_RES.fields_by_name['relation'].message_type = _CONCEPT
_THING_RELATIONS_ITER_RES.containing_type = _THING_RELATIONS_ITER
_THING_RELATIONS_ITER.containing_type = _THING_RELATIONS
_THING_RELATIONS.containing_type = _THING
_THING_ROLES_REQ.containing_type = _THING_ROLES
_THING_ROLES_ITER_RES.fields_by_name['role'].message_type = _CONCEPT
_THING_ROLES_ITER_RES.containing_type = _THING_ROLES_ITER
_THING_ROLES_ITER.containing_type = _THING_ROLES
_THING_ROLES.containing_type = _THING
_THING_RELHAS_REQ.fields_by_name['attribute'].message_type = _CONCEPT
_THING_RELHAS_REQ.containing_type = _THING_RELHAS
_THING_RELHAS_RES.fields_by_name['relation'].message_type = _CONCEPT
_THING_RELHAS_RES.containing_type = _THING_RELHAS
_THING_RELHAS.containing_type = _THING
_THING_UNHAS_REQ.fields_by_name['attribute'].message_type = _CONCEPT
_THING_UNHAS_REQ.containing_type = _THING_UNHAS
_THING_UNHAS_RES.containing_type = _THING_UNHAS
_THING_UNHAS.containing_type = _THING
_RELATION_ROLEPLAYERSMAP_REQ.containing_type = _RELATION_ROLEPLAYERSMAP
_RELATION_ROLEPLAYERSMAP_ITER_RES.fields_by_name['role'].message_type = _CONCEPT
_RELATION_ROLEPLAYERSMAP_ITER_RES.fields_by_name['player'].message_type = _CONCEPT
_RELATION_ROLEPLAYERSMAP_ITER_RES.containing_type = _RELATION_ROLEPLAYERSMAP_ITER
_RELATION_ROLEPLAYERSMAP_ITER.containing_type = _RELATION_ROLEPLAYERSMAP
_RELATION_ROLEPLAYERSMAP.containing_type = _RELATION
_RELATION_ROLEPLAYERS_REQ.fields_by_name['roles'].message_type = _CONCEPT
_RELATION_ROLEPLAYERS_REQ.containing_type = _RELATION_ROLEPLAYERS
_RELATION_ROLEPLAYERS_ITER_RES.fields_by_name['thing'].message_type = _CONCEPT
_RELATION_ROLEPLAYERS_ITER_RES.containing_type = _RELATION_ROLEPLAYERS_ITER
_RELATION_ROLEPLAYERS_ITER.containing_type = _RELATION_ROLEPLAYERS
_RELATION_ROLEPLAYERS.containing_type = _RELATION
_RELATION_ASSIGN_REQ.fields_by_name['role'].message_type = _CONCEPT
_RELATION_ASSIGN_REQ.fields_by_name['player'].message_type = _CONCEPT
_RELATION_ASSIGN_REQ.containing_type = _RELATION_ASSIGN
_RELATION_ASSIGN_RES.containing_type = _RELATION_ASSIGN
_RELATION_ASSIGN.containing_type = _RELATION
_RELATION_UNASSIGN_REQ.fields_by_name['role'].message_type = _CONCEPT
_RELATION_UNASSIGN_REQ.fields_by_name['player'].message_type = _CONCEPT
_RELATION_UNASSIGN_REQ.containing_type = _RELATION_UNASSIGN
_RELATION_UNASSIGN_RES.containing_type = _RELATION_UNASSIGN
_RELATION_UNASSIGN.containing_type = _RELATION
_ATTRIBUTE_VALUE_REQ.containing_type = _ATTRIBUTE_VALUE
_ATTRIBUTE_VALUE_RES.fields_by_name['value'].message_type = _VALUEOBJECT
_ATTRIBUTE_VALUE_RES.containing_type = _ATTRIBUTE_VALUE
_ATTRIBUTE_VALUE.containing_type = _ATTRIBUTE
_ATTRIBUTE_OWNERS_REQ.containing_type = _ATTRIBUTE_OWNERS
_ATTRIBUTE_OWNERS_ITER_RES.fields_by_name['thing'].message_type = _CONCEPT
_ATTRIBUTE_OWNERS_ITER_RES.containing_type = _ATTRIBUTE_OWNERS_ITER
_ATTRIBUTE_OWNERS_ITER.containing_type = _ATTRIBUTE_OWNERS
_ATTRIBUTE_OWNERS.containing_type = _ATTRIBUTE
_VALUEOBJECT.oneofs_by_name['value'].fields.append(
  _VALUEOBJECT.fields_by_name['string'])
_VALUEOBJECT.fields_by_name['string'].containing_oneof = _VALUEOBJECT.oneofs_by_name['value']
_VALUEOBJECT.oneofs_by_name['value'].fields.append(
  _VALUEOBJECT.fields_by_name['boolean'])
_VALUEOBJECT.fields_by_name['boolean'].containing_oneof = _VALUEOBJECT.oneofs_by_name['value']
_VALUEOBJECT.oneofs_by_name['value'].fields.append(
  _VALUEOBJECT.fields_by_name['integer'])
_VALUEOBJECT.fields_by_name['integer'].containing_oneof = _VALUEOBJECT.oneofs_by_name['value']
_VALUEOBJECT.oneofs_by_name['value'].fields.append(
  _VALUEOBJECT.fields_by_name['long'])
_VALUEOBJECT.fields_by_name['long'].containing_oneof = _VALUEOBJECT.oneofs_by_name['value']
_VALUEOBJECT.oneofs_by_name['value'].fields.append(
  _VALUEOBJECT.fields_by_name['float'])
_VALUEOBJECT.fields_by_name['float'].containing_oneof = _VALUEOBJECT.oneofs_by_name['value']
_VALUEOBJECT.oneofs_by_name['value'].fields.append(
  _VALUEOBJECT.fields_by_name['double'])
_VALUEOBJECT.fields_by_name['double'].containing_oneof = _VALUEOBJECT.oneofs_by_name['value']
_VALUEOBJECT.oneofs_by_name['value'].fields.append(
  _VALUEOBJECT.fields_by_name['date'])
_VALUEOBJECT.fields_by_name['date'].containing_oneof = _VALUEOBJECT.oneofs_by_name['value']
DESCRIPTOR.message_types_by_name['Method'] = _METHOD
DESCRIPTOR.message_types_by_name['Null'] = _NULL
DESCRIPTOR.message_types_by_name['Concept'] = _CONCEPT
DESCRIPTOR.message_types_by_name['SchemaConcept'] = _SCHEMACONCEPT
DESCRIPTOR.message_types_by_name['Rule'] = _RULE
DESCRIPTOR.message_types_by_name['Role'] = _ROLE
DESCRIPTOR.message_types_by_name['Type'] = _TYPE
DESCRIPTOR.message_types_by_name['EntityType'] = _ENTITYTYPE
DESCRIPTOR.message_types_by_name['RelationType'] = _RELATIONTYPE
DESCRIPTOR.message_types_by_name['AttributeType'] = _ATTRIBUTETYPE
DESCRIPTOR.message_types_by_name['Thing'] = _THING
DESCRIPTOR.message_types_by_name['Relation'] = _RELATION
DESCRIPTOR.message_types_by_name['Attribute'] = _ATTRIBUTE
DESCRIPTOR.message_types_by_name['ValueObject'] = _VALUEOBJECT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Method = _reflection.GeneratedProtocolMessageType('Method', (_message.Message,), dict(

  Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
    DESCRIPTOR = _METHOD_REQ,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Method.Req)
    ))
  ,

  Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
    DESCRIPTOR = _METHOD_RES,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Method.Res)
    ))
  ,

  Iter = _reflection.GeneratedProtocolMessageType('Iter', (_message.Message,), dict(

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _METHOD_ITER_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Method.Iter.Res)
      ))
    ,
    DESCRIPTOR = _METHOD_ITER,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Method.Iter)
    ))
  ,
  DESCRIPTOR = _METHOD,
  __module__ = 'session.Concept_pb2'
  # @@protoc_insertion_point(class_scope:session.Method)
  ))
_sym_db.RegisterMessage(Method)
_sym_db.RegisterMessage(Method.Req)
_sym_db.RegisterMessage(Method.Res)
_sym_db.RegisterMessage(Method.Iter)
_sym_db.RegisterMessage(Method.Iter.Res)

Null = _reflection.GeneratedProtocolMessageType('Null', (_message.Message,), dict(
  DESCRIPTOR = _NULL,
  __module__ = 'session.Concept_pb2'
  # @@protoc_insertion_point(class_scope:session.Null)
  ))
_sym_db.RegisterMessage(Null)

Concept = _reflection.GeneratedProtocolMessageType('Concept', (_message.Message,), dict(

  Delete = _reflection.GeneratedProtocolMessageType('Delete', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _CONCEPT_DELETE_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Concept.Delete.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _CONCEPT_DELETE_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Concept.Delete.Res)
      ))
    ,
    DESCRIPTOR = _CONCEPT_DELETE,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Concept.Delete)
    ))
  ,
  DESCRIPTOR = _CONCEPT,
  __module__ = 'session.Concept_pb2'
  # @@protoc_insertion_point(class_scope:session.Concept)
  ))
_sym_db.RegisterMessage(Concept)
_sym_db.RegisterMessage(Concept.Delete)
_sym_db.RegisterMessage(Concept.Delete.Req)
_sym_db.RegisterMessage(Concept.Delete.Res)

SchemaConcept = _reflection.GeneratedProtocolMessageType('SchemaConcept', (_message.Message,), dict(

  GetLabel = _reflection.GeneratedProtocolMessageType('GetLabel', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _SCHEMACONCEPT_GETLABEL_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.SchemaConcept.GetLabel.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _SCHEMACONCEPT_GETLABEL_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.SchemaConcept.GetLabel.Res)
      ))
    ,
    DESCRIPTOR = _SCHEMACONCEPT_GETLABEL,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.SchemaConcept.GetLabel)
    ))
  ,

  SetLabel = _reflection.GeneratedProtocolMessageType('SetLabel', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _SCHEMACONCEPT_SETLABEL_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.SchemaConcept.SetLabel.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _SCHEMACONCEPT_SETLABEL_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.SchemaConcept.SetLabel.Res)
      ))
    ,
    DESCRIPTOR = _SCHEMACONCEPT_SETLABEL,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.SchemaConcept.SetLabel)
    ))
  ,

  IsImplicit = _reflection.GeneratedProtocolMessageType('IsImplicit', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _SCHEMACONCEPT_ISIMPLICIT_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.SchemaConcept.IsImplicit.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _SCHEMACONCEPT_ISIMPLICIT_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.SchemaConcept.IsImplicit.Res)
      ))
    ,
    DESCRIPTOR = _SCHEMACONCEPT_ISIMPLICIT,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.SchemaConcept.IsImplicit)
    ))
  ,

  GetSup = _reflection.GeneratedProtocolMessageType('GetSup', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _SCHEMACONCEPT_GETSUP_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.SchemaConcept.GetSup.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _SCHEMACONCEPT_GETSUP_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.SchemaConcept.GetSup.Res)
      ))
    ,
    DESCRIPTOR = _SCHEMACONCEPT_GETSUP,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.SchemaConcept.GetSup)
    ))
  ,

  SetSup = _reflection.GeneratedProtocolMessageType('SetSup', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _SCHEMACONCEPT_SETSUP_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.SchemaConcept.SetSup.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _SCHEMACONCEPT_SETSUP_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.SchemaConcept.SetSup.Res)
      ))
    ,
    DESCRIPTOR = _SCHEMACONCEPT_SETSUP,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.SchemaConcept.SetSup)
    ))
  ,

  Sups = _reflection.GeneratedProtocolMessageType('Sups', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _SCHEMACONCEPT_SUPS_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.SchemaConcept.Sups.Req)
      ))
    ,

    Iter = _reflection.GeneratedProtocolMessageType('Iter', (_message.Message,), dict(

      Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
        DESCRIPTOR = _SCHEMACONCEPT_SUPS_ITER_RES,
        __module__ = 'session.Concept_pb2'
        # @@protoc_insertion_point(class_scope:session.SchemaConcept.Sups.Iter.Res)
        ))
      ,
      DESCRIPTOR = _SCHEMACONCEPT_SUPS_ITER,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.SchemaConcept.Sups.Iter)
      ))
    ,
    DESCRIPTOR = _SCHEMACONCEPT_SUPS,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.SchemaConcept.Sups)
    ))
  ,

  Subs = _reflection.GeneratedProtocolMessageType('Subs', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _SCHEMACONCEPT_SUBS_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.SchemaConcept.Subs.Req)
      ))
    ,

    Iter = _reflection.GeneratedProtocolMessageType('Iter', (_message.Message,), dict(

      Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
        DESCRIPTOR = _SCHEMACONCEPT_SUBS_ITER_RES,
        __module__ = 'session.Concept_pb2'
        # @@protoc_insertion_point(class_scope:session.SchemaConcept.Subs.Iter.Res)
        ))
      ,
      DESCRIPTOR = _SCHEMACONCEPT_SUBS_ITER,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.SchemaConcept.Subs.Iter)
      ))
    ,
    DESCRIPTOR = _SCHEMACONCEPT_SUBS,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.SchemaConcept.Subs)
    ))
  ,
  DESCRIPTOR = _SCHEMACONCEPT,
  __module__ = 'session.Concept_pb2'
  # @@protoc_insertion_point(class_scope:session.SchemaConcept)
  ))
_sym_db.RegisterMessage(SchemaConcept)
_sym_db.RegisterMessage(SchemaConcept.GetLabel)
_sym_db.RegisterMessage(SchemaConcept.GetLabel.Req)
_sym_db.RegisterMessage(SchemaConcept.GetLabel.Res)
_sym_db.RegisterMessage(SchemaConcept.SetLabel)
_sym_db.RegisterMessage(SchemaConcept.SetLabel.Req)
_sym_db.RegisterMessage(SchemaConcept.SetLabel.Res)
_sym_db.RegisterMessage(SchemaConcept.IsImplicit)
_sym_db.RegisterMessage(SchemaConcept.IsImplicit.Req)
_sym_db.RegisterMessage(SchemaConcept.IsImplicit.Res)
_sym_db.RegisterMessage(SchemaConcept.GetSup)
_sym_db.RegisterMessage(SchemaConcept.GetSup.Req)
_sym_db.RegisterMessage(SchemaConcept.GetSup.Res)
_sym_db.RegisterMessage(SchemaConcept.SetSup)
_sym_db.RegisterMessage(SchemaConcept.SetSup.Req)
_sym_db.RegisterMessage(SchemaConcept.SetSup.Res)
_sym_db.RegisterMessage(SchemaConcept.Sups)
_sym_db.RegisterMessage(SchemaConcept.Sups.Req)
_sym_db.RegisterMessage(SchemaConcept.Sups.Iter)
_sym_db.RegisterMessage(SchemaConcept.Sups.Iter.Res)
_sym_db.RegisterMessage(SchemaConcept.Subs)
_sym_db.RegisterMessage(SchemaConcept.Subs.Req)
_sym_db.RegisterMessage(SchemaConcept.Subs.Iter)
_sym_db.RegisterMessage(SchemaConcept.Subs.Iter.Res)

Rule = _reflection.GeneratedProtocolMessageType('Rule', (_message.Message,), dict(

  When = _reflection.GeneratedProtocolMessageType('When', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _RULE_WHEN_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Rule.When.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _RULE_WHEN_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Rule.When.Res)
      ))
    ,
    DESCRIPTOR = _RULE_WHEN,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Rule.When)
    ))
  ,

  Then = _reflection.GeneratedProtocolMessageType('Then', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _RULE_THEN_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Rule.Then.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _RULE_THEN_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Rule.Then.Res)
      ))
    ,
    DESCRIPTOR = _RULE_THEN,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Rule.Then)
    ))
  ,
  DESCRIPTOR = _RULE,
  __module__ = 'session.Concept_pb2'
  # @@protoc_insertion_point(class_scope:session.Rule)
  ))
_sym_db.RegisterMessage(Rule)
_sym_db.RegisterMessage(Rule.When)
_sym_db.RegisterMessage(Rule.When.Req)
_sym_db.RegisterMessage(Rule.When.Res)
_sym_db.RegisterMessage(Rule.Then)
_sym_db.RegisterMessage(Rule.Then.Req)
_sym_db.RegisterMessage(Rule.Then.Res)

Role = _reflection.GeneratedProtocolMessageType('Role', (_message.Message,), dict(

  Relations = _reflection.GeneratedProtocolMessageType('Relations', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _ROLE_RELATIONS_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Role.Relations.Req)
      ))
    ,

    Iter = _reflection.GeneratedProtocolMessageType('Iter', (_message.Message,), dict(

      Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
        DESCRIPTOR = _ROLE_RELATIONS_ITER_RES,
        __module__ = 'session.Concept_pb2'
        # @@protoc_insertion_point(class_scope:session.Role.Relations.Iter.Res)
        ))
      ,
      DESCRIPTOR = _ROLE_RELATIONS_ITER,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Role.Relations.Iter)
      ))
    ,
    DESCRIPTOR = _ROLE_RELATIONS,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Role.Relations)
    ))
  ,

  Players = _reflection.GeneratedProtocolMessageType('Players', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _ROLE_PLAYERS_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Role.Players.Req)
      ))
    ,

    Iter = _reflection.GeneratedProtocolMessageType('Iter', (_message.Message,), dict(

      Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
        DESCRIPTOR = _ROLE_PLAYERS_ITER_RES,
        __module__ = 'session.Concept_pb2'
        # @@protoc_insertion_point(class_scope:session.Role.Players.Iter.Res)
        ))
      ,
      DESCRIPTOR = _ROLE_PLAYERS_ITER,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Role.Players.Iter)
      ))
    ,
    DESCRIPTOR = _ROLE_PLAYERS,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Role.Players)
    ))
  ,
  DESCRIPTOR = _ROLE,
  __module__ = 'session.Concept_pb2'
  # @@protoc_insertion_point(class_scope:session.Role)
  ))
_sym_db.RegisterMessage(Role)
_sym_db.RegisterMessage(Role.Relations)
_sym_db.RegisterMessage(Role.Relations.Req)
_sym_db.RegisterMessage(Role.Relations.Iter)
_sym_db.RegisterMessage(Role.Relations.Iter.Res)
_sym_db.RegisterMessage(Role.Players)
_sym_db.RegisterMessage(Role.Players.Req)
_sym_db.RegisterMessage(Role.Players.Iter)
_sym_db.RegisterMessage(Role.Players.Iter.Res)

Type = _reflection.GeneratedProtocolMessageType('Type', (_message.Message,), dict(

  IsAbstract = _reflection.GeneratedProtocolMessageType('IsAbstract', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _TYPE_ISABSTRACT_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Type.IsAbstract.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _TYPE_ISABSTRACT_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Type.IsAbstract.Res)
      ))
    ,
    DESCRIPTOR = _TYPE_ISABSTRACT,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Type.IsAbstract)
    ))
  ,

  SetAbstract = _reflection.GeneratedProtocolMessageType('SetAbstract', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _TYPE_SETABSTRACT_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Type.SetAbstract.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _TYPE_SETABSTRACT_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Type.SetAbstract.Res)
      ))
    ,
    DESCRIPTOR = _TYPE_SETABSTRACT,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Type.SetAbstract)
    ))
  ,

  Instances = _reflection.GeneratedProtocolMessageType('Instances', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _TYPE_INSTANCES_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Type.Instances.Req)
      ))
    ,

    Iter = _reflection.GeneratedProtocolMessageType('Iter', (_message.Message,), dict(

      Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
        DESCRIPTOR = _TYPE_INSTANCES_ITER_RES,
        __module__ = 'session.Concept_pb2'
        # @@protoc_insertion_point(class_scope:session.Type.Instances.Iter.Res)
        ))
      ,
      DESCRIPTOR = _TYPE_INSTANCES_ITER,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Type.Instances.Iter)
      ))
    ,
    DESCRIPTOR = _TYPE_INSTANCES,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Type.Instances)
    ))
  ,

  Attributes = _reflection.GeneratedProtocolMessageType('Attributes', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _TYPE_ATTRIBUTES_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Type.Attributes.Req)
      ))
    ,

    Iter = _reflection.GeneratedProtocolMessageType('Iter', (_message.Message,), dict(

      Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
        DESCRIPTOR = _TYPE_ATTRIBUTES_ITER_RES,
        __module__ = 'session.Concept_pb2'
        # @@protoc_insertion_point(class_scope:session.Type.Attributes.Iter.Res)
        ))
      ,
      DESCRIPTOR = _TYPE_ATTRIBUTES_ITER,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Type.Attributes.Iter)
      ))
    ,
    DESCRIPTOR = _TYPE_ATTRIBUTES,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Type.Attributes)
    ))
  ,

  Keys = _reflection.GeneratedProtocolMessageType('Keys', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _TYPE_KEYS_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Type.Keys.Req)
      ))
    ,

    Iter = _reflection.GeneratedProtocolMessageType('Iter', (_message.Message,), dict(

      Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
        DESCRIPTOR = _TYPE_KEYS_ITER_RES,
        __module__ = 'session.Concept_pb2'
        # @@protoc_insertion_point(class_scope:session.Type.Keys.Iter.Res)
        ))
      ,
      DESCRIPTOR = _TYPE_KEYS_ITER,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Type.Keys.Iter)
      ))
    ,
    DESCRIPTOR = _TYPE_KEYS,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Type.Keys)
    ))
  ,

  Playing = _reflection.GeneratedProtocolMessageType('Playing', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _TYPE_PLAYING_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Type.Playing.Req)
      ))
    ,

    Iter = _reflection.GeneratedProtocolMessageType('Iter', (_message.Message,), dict(

      Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
        DESCRIPTOR = _TYPE_PLAYING_ITER_RES,
        __module__ = 'session.Concept_pb2'
        # @@protoc_insertion_point(class_scope:session.Type.Playing.Iter.Res)
        ))
      ,
      DESCRIPTOR = _TYPE_PLAYING_ITER,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Type.Playing.Iter)
      ))
    ,
    DESCRIPTOR = _TYPE_PLAYING,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Type.Playing)
    ))
  ,

  Key = _reflection.GeneratedProtocolMessageType('Key', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _TYPE_KEY_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Type.Key.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _TYPE_KEY_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Type.Key.Res)
      ))
    ,
    DESCRIPTOR = _TYPE_KEY,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Type.Key)
    ))
  ,

  Has = _reflection.GeneratedProtocolMessageType('Has', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _TYPE_HAS_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Type.Has.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _TYPE_HAS_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Type.Has.Res)
      ))
    ,
    DESCRIPTOR = _TYPE_HAS,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Type.Has)
    ))
  ,

  Plays = _reflection.GeneratedProtocolMessageType('Plays', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _TYPE_PLAYS_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Type.Plays.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _TYPE_PLAYS_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Type.Plays.Res)
      ))
    ,
    DESCRIPTOR = _TYPE_PLAYS,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Type.Plays)
    ))
  ,

  Unkey = _reflection.GeneratedProtocolMessageType('Unkey', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _TYPE_UNKEY_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Type.Unkey.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _TYPE_UNKEY_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Type.Unkey.Res)
      ))
    ,
    DESCRIPTOR = _TYPE_UNKEY,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Type.Unkey)
    ))
  ,

  Unhas = _reflection.GeneratedProtocolMessageType('Unhas', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _TYPE_UNHAS_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Type.Unhas.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _TYPE_UNHAS_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Type.Unhas.Res)
      ))
    ,
    DESCRIPTOR = _TYPE_UNHAS,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Type.Unhas)
    ))
  ,

  Unplay = _reflection.GeneratedProtocolMessageType('Unplay', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _TYPE_UNPLAY_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Type.Unplay.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _TYPE_UNPLAY_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Type.Unplay.Res)
      ))
    ,
    DESCRIPTOR = _TYPE_UNPLAY,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Type.Unplay)
    ))
  ,
  DESCRIPTOR = _TYPE,
  __module__ = 'session.Concept_pb2'
  # @@protoc_insertion_point(class_scope:session.Type)
  ))
_sym_db.RegisterMessage(Type)
_sym_db.RegisterMessage(Type.IsAbstract)
_sym_db.RegisterMessage(Type.IsAbstract.Req)
_sym_db.RegisterMessage(Type.IsAbstract.Res)
_sym_db.RegisterMessage(Type.SetAbstract)
_sym_db.RegisterMessage(Type.SetAbstract.Req)
_sym_db.RegisterMessage(Type.SetAbstract.Res)
_sym_db.RegisterMessage(Type.Instances)
_sym_db.RegisterMessage(Type.Instances.Req)
_sym_db.RegisterMessage(Type.Instances.Iter)
_sym_db.RegisterMessage(Type.Instances.Iter.Res)
_sym_db.RegisterMessage(Type.Attributes)
_sym_db.RegisterMessage(Type.Attributes.Req)
_sym_db.RegisterMessage(Type.Attributes.Iter)
_sym_db.RegisterMessage(Type.Attributes.Iter.Res)
_sym_db.RegisterMessage(Type.Keys)
_sym_db.RegisterMessage(Type.Keys.Req)
_sym_db.RegisterMessage(Type.Keys.Iter)
_sym_db.RegisterMessage(Type.Keys.Iter.Res)
_sym_db.RegisterMessage(Type.Playing)
_sym_db.RegisterMessage(Type.Playing.Req)
_sym_db.RegisterMessage(Type.Playing.Iter)
_sym_db.RegisterMessage(Type.Playing.Iter.Res)
_sym_db.RegisterMessage(Type.Key)
_sym_db.RegisterMessage(Type.Key.Req)
_sym_db.RegisterMessage(Type.Key.Res)
_sym_db.RegisterMessage(Type.Has)
_sym_db.RegisterMessage(Type.Has.Req)
_sym_db.RegisterMessage(Type.Has.Res)
_sym_db.RegisterMessage(Type.Plays)
_sym_db.RegisterMessage(Type.Plays.Req)
_sym_db.RegisterMessage(Type.Plays.Res)
_sym_db.RegisterMessage(Type.Unkey)
_sym_db.RegisterMessage(Type.Unkey.Req)
_sym_db.RegisterMessage(Type.Unkey.Res)
_sym_db.RegisterMessage(Type.Unhas)
_sym_db.RegisterMessage(Type.Unhas.Req)
_sym_db.RegisterMessage(Type.Unhas.Res)
_sym_db.RegisterMessage(Type.Unplay)
_sym_db.RegisterMessage(Type.Unplay.Req)
_sym_db.RegisterMessage(Type.Unplay.Res)

EntityType = _reflection.GeneratedProtocolMessageType('EntityType', (_message.Message,), dict(

  Create = _reflection.GeneratedProtocolMessageType('Create', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _ENTITYTYPE_CREATE_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.EntityType.Create.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _ENTITYTYPE_CREATE_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.EntityType.Create.Res)
      ))
    ,
    DESCRIPTOR = _ENTITYTYPE_CREATE,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.EntityType.Create)
    ))
  ,
  DESCRIPTOR = _ENTITYTYPE,
  __module__ = 'session.Concept_pb2'
  # @@protoc_insertion_point(class_scope:session.EntityType)
  ))
_sym_db.RegisterMessage(EntityType)
_sym_db.RegisterMessage(EntityType.Create)
_sym_db.RegisterMessage(EntityType.Create.Req)
_sym_db.RegisterMessage(EntityType.Create.Res)

RelationType = _reflection.GeneratedProtocolMessageType('RelationType', (_message.Message,), dict(

  Create = _reflection.GeneratedProtocolMessageType('Create', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _RELATIONTYPE_CREATE_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.RelationType.Create.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _RELATIONTYPE_CREATE_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.RelationType.Create.Res)
      ))
    ,
    DESCRIPTOR = _RELATIONTYPE_CREATE,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.RelationType.Create)
    ))
  ,

  Roles = _reflection.GeneratedProtocolMessageType('Roles', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _RELATIONTYPE_ROLES_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.RelationType.Roles.Req)
      ))
    ,

    Iter = _reflection.GeneratedProtocolMessageType('Iter', (_message.Message,), dict(

      Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
        DESCRIPTOR = _RELATIONTYPE_ROLES_ITER_RES,
        __module__ = 'session.Concept_pb2'
        # @@protoc_insertion_point(class_scope:session.RelationType.Roles.Iter.Res)
        ))
      ,
      DESCRIPTOR = _RELATIONTYPE_ROLES_ITER,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.RelationType.Roles.Iter)
      ))
    ,
    DESCRIPTOR = _RELATIONTYPE_ROLES,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.RelationType.Roles)
    ))
  ,

  Relates = _reflection.GeneratedProtocolMessageType('Relates', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _RELATIONTYPE_RELATES_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.RelationType.Relates.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _RELATIONTYPE_RELATES_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.RelationType.Relates.Res)
      ))
    ,
    DESCRIPTOR = _RELATIONTYPE_RELATES,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.RelationType.Relates)
    ))
  ,

  Unrelate = _reflection.GeneratedProtocolMessageType('Unrelate', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _RELATIONTYPE_UNRELATE_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.RelationType.Unrelate.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _RELATIONTYPE_UNRELATE_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.RelationType.Unrelate.Res)
      ))
    ,
    DESCRIPTOR = _RELATIONTYPE_UNRELATE,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.RelationType.Unrelate)
    ))
  ,
  DESCRIPTOR = _RELATIONTYPE,
  __module__ = 'session.Concept_pb2'
  # @@protoc_insertion_point(class_scope:session.RelationType)
  ))
_sym_db.RegisterMessage(RelationType)
_sym_db.RegisterMessage(RelationType.Create)
_sym_db.RegisterMessage(RelationType.Create.Req)
_sym_db.RegisterMessage(RelationType.Create.Res)
_sym_db.RegisterMessage(RelationType.Roles)
_sym_db.RegisterMessage(RelationType.Roles.Req)
_sym_db.RegisterMessage(RelationType.Roles.Iter)
_sym_db.RegisterMessage(RelationType.Roles.Iter.Res)
_sym_db.RegisterMessage(RelationType.Relates)
_sym_db.RegisterMessage(RelationType.Relates.Req)
_sym_db.RegisterMessage(RelationType.Relates.Res)
_sym_db.RegisterMessage(RelationType.Unrelate)
_sym_db.RegisterMessage(RelationType.Unrelate.Req)
_sym_db.RegisterMessage(RelationType.Unrelate.Res)

AttributeType = _reflection.GeneratedProtocolMessageType('AttributeType', (_message.Message,), dict(

  Create = _reflection.GeneratedProtocolMessageType('Create', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _ATTRIBUTETYPE_CREATE_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.AttributeType.Create.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _ATTRIBUTETYPE_CREATE_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.AttributeType.Create.Res)
      ))
    ,
    DESCRIPTOR = _ATTRIBUTETYPE_CREATE,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.AttributeType.Create)
    ))
  ,

  Attribute = _reflection.GeneratedProtocolMessageType('Attribute', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _ATTRIBUTETYPE_ATTRIBUTE_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.AttributeType.Attribute.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _ATTRIBUTETYPE_ATTRIBUTE_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.AttributeType.Attribute.Res)
      ))
    ,
    DESCRIPTOR = _ATTRIBUTETYPE_ATTRIBUTE,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.AttributeType.Attribute)
    ))
  ,

  DataType = _reflection.GeneratedProtocolMessageType('DataType', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _ATTRIBUTETYPE_DATATYPE_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.AttributeType.DataType.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _ATTRIBUTETYPE_DATATYPE_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.AttributeType.DataType.Res)
      ))
    ,
    DESCRIPTOR = _ATTRIBUTETYPE_DATATYPE,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.AttributeType.DataType)
    ))
  ,

  GetRegex = _reflection.GeneratedProtocolMessageType('GetRegex', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _ATTRIBUTETYPE_GETREGEX_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.AttributeType.GetRegex.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _ATTRIBUTETYPE_GETREGEX_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.AttributeType.GetRegex.Res)
      ))
    ,
    DESCRIPTOR = _ATTRIBUTETYPE_GETREGEX,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.AttributeType.GetRegex)
    ))
  ,

  SetRegex = _reflection.GeneratedProtocolMessageType('SetRegex', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _ATTRIBUTETYPE_SETREGEX_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.AttributeType.SetRegex.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _ATTRIBUTETYPE_SETREGEX_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.AttributeType.SetRegex.Res)
      ))
    ,
    DESCRIPTOR = _ATTRIBUTETYPE_SETREGEX,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.AttributeType.SetRegex)
    ))
  ,
  DESCRIPTOR = _ATTRIBUTETYPE,
  __module__ = 'session.Concept_pb2'
  # @@protoc_insertion_point(class_scope:session.AttributeType)
  ))
_sym_db.RegisterMessage(AttributeType)
_sym_db.RegisterMessage(AttributeType.Create)
_sym_db.RegisterMessage(AttributeType.Create.Req)
_sym_db.RegisterMessage(AttributeType.Create.Res)
_sym_db.RegisterMessage(AttributeType.Attribute)
_sym_db.RegisterMessage(AttributeType.Attribute.Req)
_sym_db.RegisterMessage(AttributeType.Attribute.Res)
_sym_db.RegisterMessage(AttributeType.DataType)
_sym_db.RegisterMessage(AttributeType.DataType.Req)
_sym_db.RegisterMessage(AttributeType.DataType.Res)
_sym_db.RegisterMessage(AttributeType.GetRegex)
_sym_db.RegisterMessage(AttributeType.GetRegex.Req)
_sym_db.RegisterMessage(AttributeType.GetRegex.Res)
_sym_db.RegisterMessage(AttributeType.SetRegex)
_sym_db.RegisterMessage(AttributeType.SetRegex.Req)
_sym_db.RegisterMessage(AttributeType.SetRegex.Res)

Thing = _reflection.GeneratedProtocolMessageType('Thing', (_message.Message,), dict(

  IsInferred = _reflection.GeneratedProtocolMessageType('IsInferred', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _THING_ISINFERRED_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Thing.IsInferred.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _THING_ISINFERRED_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Thing.IsInferred.Res)
      ))
    ,
    DESCRIPTOR = _THING_ISINFERRED,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Thing.IsInferred)
    ))
  ,

  Type = _reflection.GeneratedProtocolMessageType('Type', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _THING_TYPE_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Thing.Type.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _THING_TYPE_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Thing.Type.Res)
      ))
    ,
    DESCRIPTOR = _THING_TYPE,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Thing.Type)
    ))
  ,

  Keys = _reflection.GeneratedProtocolMessageType('Keys', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _THING_KEYS_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Thing.Keys.Req)
      ))
    ,

    Iter = _reflection.GeneratedProtocolMessageType('Iter', (_message.Message,), dict(

      Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
        DESCRIPTOR = _THING_KEYS_ITER_RES,
        __module__ = 'session.Concept_pb2'
        # @@protoc_insertion_point(class_scope:session.Thing.Keys.Iter.Res)
        ))
      ,
      DESCRIPTOR = _THING_KEYS_ITER,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Thing.Keys.Iter)
      ))
    ,
    DESCRIPTOR = _THING_KEYS,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Thing.Keys)
    ))
  ,

  Attributes = _reflection.GeneratedProtocolMessageType('Attributes', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _THING_ATTRIBUTES_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Thing.Attributes.Req)
      ))
    ,

    Iter = _reflection.GeneratedProtocolMessageType('Iter', (_message.Message,), dict(

      Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
        DESCRIPTOR = _THING_ATTRIBUTES_ITER_RES,
        __module__ = 'session.Concept_pb2'
        # @@protoc_insertion_point(class_scope:session.Thing.Attributes.Iter.Res)
        ))
      ,
      DESCRIPTOR = _THING_ATTRIBUTES_ITER,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Thing.Attributes.Iter)
      ))
    ,
    DESCRIPTOR = _THING_ATTRIBUTES,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Thing.Attributes)
    ))
  ,

  Relations = _reflection.GeneratedProtocolMessageType('Relations', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _THING_RELATIONS_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Thing.Relations.Req)
      ))
    ,

    Iter = _reflection.GeneratedProtocolMessageType('Iter', (_message.Message,), dict(

      Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
        DESCRIPTOR = _THING_RELATIONS_ITER_RES,
        __module__ = 'session.Concept_pb2'
        # @@protoc_insertion_point(class_scope:session.Thing.Relations.Iter.Res)
        ))
      ,
      DESCRIPTOR = _THING_RELATIONS_ITER,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Thing.Relations.Iter)
      ))
    ,
    DESCRIPTOR = _THING_RELATIONS,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Thing.Relations)
    ))
  ,

  Roles = _reflection.GeneratedProtocolMessageType('Roles', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _THING_ROLES_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Thing.Roles.Req)
      ))
    ,

    Iter = _reflection.GeneratedProtocolMessageType('Iter', (_message.Message,), dict(

      Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
        DESCRIPTOR = _THING_ROLES_ITER_RES,
        __module__ = 'session.Concept_pb2'
        # @@protoc_insertion_point(class_scope:session.Thing.Roles.Iter.Res)
        ))
      ,
      DESCRIPTOR = _THING_ROLES_ITER,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Thing.Roles.Iter)
      ))
    ,
    DESCRIPTOR = _THING_ROLES,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Thing.Roles)
    ))
  ,

  Relhas = _reflection.GeneratedProtocolMessageType('Relhas', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _THING_RELHAS_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Thing.Relhas.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _THING_RELHAS_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Thing.Relhas.Res)
      ))
    ,
    DESCRIPTOR = _THING_RELHAS,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Thing.Relhas)
    ))
  ,

  Unhas = _reflection.GeneratedProtocolMessageType('Unhas', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _THING_UNHAS_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Thing.Unhas.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _THING_UNHAS_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Thing.Unhas.Res)
      ))
    ,
    DESCRIPTOR = _THING_UNHAS,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Thing.Unhas)
    ))
  ,
  DESCRIPTOR = _THING,
  __module__ = 'session.Concept_pb2'
  # @@protoc_insertion_point(class_scope:session.Thing)
  ))
_sym_db.RegisterMessage(Thing)
_sym_db.RegisterMessage(Thing.IsInferred)
_sym_db.RegisterMessage(Thing.IsInferred.Req)
_sym_db.RegisterMessage(Thing.IsInferred.Res)
_sym_db.RegisterMessage(Thing.Type)
_sym_db.RegisterMessage(Thing.Type.Req)
_sym_db.RegisterMessage(Thing.Type.Res)
_sym_db.RegisterMessage(Thing.Keys)
_sym_db.RegisterMessage(Thing.Keys.Req)
_sym_db.RegisterMessage(Thing.Keys.Iter)
_sym_db.RegisterMessage(Thing.Keys.Iter.Res)
_sym_db.RegisterMessage(Thing.Attributes)
_sym_db.RegisterMessage(Thing.Attributes.Req)
_sym_db.RegisterMessage(Thing.Attributes.Iter)
_sym_db.RegisterMessage(Thing.Attributes.Iter.Res)
_sym_db.RegisterMessage(Thing.Relations)
_sym_db.RegisterMessage(Thing.Relations.Req)
_sym_db.RegisterMessage(Thing.Relations.Iter)
_sym_db.RegisterMessage(Thing.Relations.Iter.Res)
_sym_db.RegisterMessage(Thing.Roles)
_sym_db.RegisterMessage(Thing.Roles.Req)
_sym_db.RegisterMessage(Thing.Roles.Iter)
_sym_db.RegisterMessage(Thing.Roles.Iter.Res)
_sym_db.RegisterMessage(Thing.Relhas)
_sym_db.RegisterMessage(Thing.Relhas.Req)
_sym_db.RegisterMessage(Thing.Relhas.Res)
_sym_db.RegisterMessage(Thing.Unhas)
_sym_db.RegisterMessage(Thing.Unhas.Req)
_sym_db.RegisterMessage(Thing.Unhas.Res)

Relation = _reflection.GeneratedProtocolMessageType('Relation', (_message.Message,), dict(

  RolePlayersMap = _reflection.GeneratedProtocolMessageType('RolePlayersMap', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _RELATION_ROLEPLAYERSMAP_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Relation.RolePlayersMap.Req)
      ))
    ,

    Iter = _reflection.GeneratedProtocolMessageType('Iter', (_message.Message,), dict(

      Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
        DESCRIPTOR = _RELATION_ROLEPLAYERSMAP_ITER_RES,
        __module__ = 'session.Concept_pb2'
        # @@protoc_insertion_point(class_scope:session.Relation.RolePlayersMap.Iter.Res)
        ))
      ,
      DESCRIPTOR = _RELATION_ROLEPLAYERSMAP_ITER,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Relation.RolePlayersMap.Iter)
      ))
    ,
    DESCRIPTOR = _RELATION_ROLEPLAYERSMAP,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Relation.RolePlayersMap)
    ))
  ,

  RolePlayers = _reflection.GeneratedProtocolMessageType('RolePlayers', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _RELATION_ROLEPLAYERS_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Relation.RolePlayers.Req)
      ))
    ,

    Iter = _reflection.GeneratedProtocolMessageType('Iter', (_message.Message,), dict(

      Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
        DESCRIPTOR = _RELATION_ROLEPLAYERS_ITER_RES,
        __module__ = 'session.Concept_pb2'
        # @@protoc_insertion_point(class_scope:session.Relation.RolePlayers.Iter.Res)
        ))
      ,
      DESCRIPTOR = _RELATION_ROLEPLAYERS_ITER,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Relation.RolePlayers.Iter)
      ))
    ,
    DESCRIPTOR = _RELATION_ROLEPLAYERS,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Relation.RolePlayers)
    ))
  ,

  Assign = _reflection.GeneratedProtocolMessageType('Assign', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _RELATION_ASSIGN_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Relation.Assign.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _RELATION_ASSIGN_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Relation.Assign.Res)
      ))
    ,
    DESCRIPTOR = _RELATION_ASSIGN,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Relation.Assign)
    ))
  ,

  Unassign = _reflection.GeneratedProtocolMessageType('Unassign', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _RELATION_UNASSIGN_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Relation.Unassign.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _RELATION_UNASSIGN_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Relation.Unassign.Res)
      ))
    ,
    DESCRIPTOR = _RELATION_UNASSIGN,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Relation.Unassign)
    ))
  ,
  DESCRIPTOR = _RELATION,
  __module__ = 'session.Concept_pb2'
  # @@protoc_insertion_point(class_scope:session.Relation)
  ))
_sym_db.RegisterMessage(Relation)
_sym_db.RegisterMessage(Relation.RolePlayersMap)
_sym_db.RegisterMessage(Relation.RolePlayersMap.Req)
_sym_db.RegisterMessage(Relation.RolePlayersMap.Iter)
_sym_db.RegisterMessage(Relation.RolePlayersMap.Iter.Res)
_sym_db.RegisterMessage(Relation.RolePlayers)
_sym_db.RegisterMessage(Relation.RolePlayers.Req)
_sym_db.RegisterMessage(Relation.RolePlayers.Iter)
_sym_db.RegisterMessage(Relation.RolePlayers.Iter.Res)
_sym_db.RegisterMessage(Relation.Assign)
_sym_db.RegisterMessage(Relation.Assign.Req)
_sym_db.RegisterMessage(Relation.Assign.Res)
_sym_db.RegisterMessage(Relation.Unassign)
_sym_db.RegisterMessage(Relation.Unassign.Req)
_sym_db.RegisterMessage(Relation.Unassign.Res)

Attribute = _reflection.GeneratedProtocolMessageType('Attribute', (_message.Message,), dict(

  Value = _reflection.GeneratedProtocolMessageType('Value', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _ATTRIBUTE_VALUE_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Attribute.Value.Req)
      ))
    ,

    Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
      DESCRIPTOR = _ATTRIBUTE_VALUE_RES,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Attribute.Value.Res)
      ))
    ,
    DESCRIPTOR = _ATTRIBUTE_VALUE,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Attribute.Value)
    ))
  ,

  Owners = _reflection.GeneratedProtocolMessageType('Owners', (_message.Message,), dict(

    Req = _reflection.GeneratedProtocolMessageType('Req', (_message.Message,), dict(
      DESCRIPTOR = _ATTRIBUTE_OWNERS_REQ,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Attribute.Owners.Req)
      ))
    ,

    Iter = _reflection.GeneratedProtocolMessageType('Iter', (_message.Message,), dict(

      Res = _reflection.GeneratedProtocolMessageType('Res', (_message.Message,), dict(
        DESCRIPTOR = _ATTRIBUTE_OWNERS_ITER_RES,
        __module__ = 'session.Concept_pb2'
        # @@protoc_insertion_point(class_scope:session.Attribute.Owners.Iter.Res)
        ))
      ,
      DESCRIPTOR = _ATTRIBUTE_OWNERS_ITER,
      __module__ = 'session.Concept_pb2'
      # @@protoc_insertion_point(class_scope:session.Attribute.Owners.Iter)
      ))
    ,
    DESCRIPTOR = _ATTRIBUTE_OWNERS,
    __module__ = 'session.Concept_pb2'
    # @@protoc_insertion_point(class_scope:session.Attribute.Owners)
    ))
  ,
  DESCRIPTOR = _ATTRIBUTE,
  __module__ = 'session.Concept_pb2'
  # @@protoc_insertion_point(class_scope:session.Attribute)
  ))
_sym_db.RegisterMessage(Attribute)
_sym_db.RegisterMessage(Attribute.Value)
_sym_db.RegisterMessage(Attribute.Value.Req)
_sym_db.RegisterMessage(Attribute.Value.Res)
_sym_db.RegisterMessage(Attribute.Owners)
_sym_db.RegisterMessage(Attribute.Owners.Req)
_sym_db.RegisterMessage(Attribute.Owners.Iter)
_sym_db.RegisterMessage(Attribute.Owners.Iter.Res)

ValueObject = _reflection.GeneratedProtocolMessageType('ValueObject', (_message.Message,), dict(
  DESCRIPTOR = _VALUEOBJECT,
  __module__ = 'session.Concept_pb2'
  # @@protoc_insertion_point(class_scope:session.ValueObject)
  ))
_sym_db.RegisterMessage(ValueObject)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
